//  This code was derived from the code appearing at
//  http://www.srrb.noaa.gov/highlights/sunrise/sunrise.html

// isLeapYear returns 1 if the 4-digit yr is a leap year, 0 if it is not
function isLeapYear(yr) {
  return ((yr % 4 === 0 && yr % 100 !== 0) || yr % 400 === 0);
}

function isNumber(inputVal) {
  var oneDecimal = false;
  var inputStr = "" + inputVal;
  for (var i = 0; i < inputStr.length; i++) {
    var oneChar = inputStr.charAt(i);
    if (i === 0 && (oneChar == "-" || oneChar == "+")) {
      continue;
    }
    if (oneChar == "." && !oneDecimal) {
      oneDecimal = true;
      continue;
    }
    if (oneChar < "0" || oneChar > "9") {
      return false;
    }
  }
  return true;
}

// convLatLong converts any type of lat/long input
// into  the table form and then handles bad input
// it is nested in the calcSun function.
function convLatLong(lonDeg, lonMin, lonSec, latDeg, latMin, latSec) {
  var latSeconds;
  if (latDeg >= 0) {
    latSeconds = (latDeg)*3600 + latMin*60 + latSec*1;

    latDeg = Math.floor(latSeconds/3600);
    latMin = Math.floor((latSeconds - (latDeg*3600))/60);
    latSec = Math.floor((latSeconds - (latDeg*3600) - (latMin*60)) + 0.5);
  }
  else if (latDeg > -1) {
    latSeconds = latDeg*3600 - latMin*60 - latSec*1;

    latDeg = "-0";
    latMin = Math.floor((-latSeconds)/60);
    latSec = Math.floor( (-latSeconds - (latMin*60)) + 0.5);
  }
  else {
    latSeconds = latDeg*3600 - latMin*60 - latSec*1;

    latDeg = Math.ceil(latSeconds/3600);
    latMin = Math.floor((-latSeconds + (latDeg*3600))/60);
    latSec = Math.floor((-latSeconds + (latDeg*3600) - (latMin*60)) + 0.5);
  }

  var lonSeconds;
  if (lonDeg >= 0) {
    lonSeconds = lonDeg*3600 + lonMin*60 + lonSec*1;
    lonDeg = Math.floor(lonSeconds/3600);
    lonMin = Math.floor((lonSeconds - (lonDeg*3600))/60);
    lonSec = Math.floor((lonSeconds - (lonDeg*3600) - (lonMin)*60) + 0.5);
  }
  else if (lonDeg > -1) {
    lonSeconds = lonDeg*3600 - lonMin*60 - lonSec*1;

    lonDeg = "-0";
    lonMin = Math.floor((-lonSeconds)/60);
    lonSec = Math.floor((-lonSeconds - (lonMin*60)) + 0.5);
  }
  else {
    lonSeconds = lonDeg*3600 - lonMin*60 - lonSec*1;
    lonDeg = Math.ceil(lonSeconds/3600);
    lonMin = Math.floor((-lonSeconds + (lonDeg*3600))/60);
    lonSec = Math.floor((-lonSeconds + (lonDeg*3600) - (lonMin*60)) + 0.5);
  }

  //Test for invalid lat/long input
  if (latSeconds > 324000) {
    latDeg = 89.8;
    latMin = 0;
    latSec = 0;
  }
  if (latSeconds < -324000) {
    latDeg = -89.8;
    latMin = 0;
    latSec = 0;
  }
  if (lonSeconds > 648000) {
    lonDeg = 180;
    lonMin = 0;
    lonSec = 0;
  }
  if (lonSeconds < -648000) {
    lonDeg = -180;
    lonMin = 0;
    lonSec =0;
  }
}

// Convert radian angle to degrees
function radToDeg(angleRad) {
  return (180.0 * angleRad / Math.PI);
}

// Convert degree angle to radians
function degToRad(angleDeg) {
  return (Math.PI * angleDeg / 180.0);
}

//***********************************************************************/
//* Name:    calcDayOfYear                                              */
//* Type:    Function                                                   */
//* Purpose: Finds numerical day-of-year from mn, day and lp year info  */
//* Argumentskey: "value"                                               */
//*   month: January = 1                                                */
//*   day  : 1 - 31                                                     */
//*   lpyr : 1 if leap year, 0 if not                                   */
//* Return value:                                                       */
//*   The numerical day of year                                         */
//***********************************************************************/

function calcDayOfYear(mn, dy, lpyr) {
  var k = (lpyr ? 1 : 2);
  var doy = Math.floor((275 * mn)/9) - k * Math.floor((mn + 9)/12) + dy -30;
  return doy;
}

//***********************************************************************/
//* Name:    calcJD                                                     */
//* Type:    Function                                                   */
//* Purpose: Julian day from calendar day                               */
//* Arguments:                                                          */
//*   year : 4 digit year                                               */
//*   month: January = 1                                                */
//*   day  : 1 - 31                                                     */
//* Return value:                                                       */
//*   The Julian day corresponding to the date                          */
//* Note:                                                               */
//*   Number is returned for start of day.  Fractional days should be   */
//*   added later.                                                      */
//***********************************************************************/

function calcJD(year, month, day) {
  if (month <= 2) {
    year -= 1;
    month += 12;
  }
  var A = Math.floor(year/100);
  var B = 2 - A + Math.floor(A/4);

  var JD = Math.floor(365.25*(year + 4716)) + Math.floor(30.6001*(month+1)) + day + B - 1524.5;
  return JD;
}

//***********************************************************************/
//* Name:    calcTimeJulianCent                                         */
//* Type:    Function                                                   */
//* Purpose: convert Julian Day to centuries since J2000.0.             */
//* Arguments:                                                          */
//*   jd : the Julian Day to convert                                    */
//* Return value:                                                       */
//*   the T value corresponding to the Julian Day                       */
//***********************************************************************/

function calcTimeJulianCent(jd) {
  var T = (jd - 2451545.0)/36525.0;
  return T;
}

//***********************************************************************/
//* Name:    calcJDFromJulianCent                                       */
//* Type:    Function                                                   */
//* Purpose: convert centuries since J2000.0 to Julian Day.             */
//* Arguments:                                                          */
//*   t : number of Julian centuries since J2000.0                      */
//* Return value:                                                       */
//*   the Julian Day corresponding to the t value                       */
//***********************************************************************/

function calcJDFromJulianCent(t) {
  var JD = t * 36525.0 + 2451545.0;
  return JD;
}

//***********************************************************************/
//* Name:    calGeomMeanLongSun                                         */
//* Type:    Function                                                   */
//* Purpose: calculate the Geometric Mean Longitude of the Sun          */
//* Arguments:                                                          */
//*   t : number of Julian centuries since J2000.0                      */
//* Return value:                                                       */
//*   the Geometric Mean Longitude of the Sun in degrees                */
//***********************************************************************/

function calcGeomMeanLongSun(t) {
  var L0 = 280.46646 + t * (36000.76983 + 0.0003032 * t);
  while(L0 > 360.0) {
    L0 -= 360.0;
  }
  while(L0 < 0.0) {
    L0 += 360.0;
  }
  return L0;      // in degrees
}

//***********************************************************************/
//* Name:    calGeomAnomalySun                                          */
//* Type:    Function                                                   */
//* Purpose: calculate the Geometric Mean Anomaly of the Sun            */
//* Arguments:                                                          */
//*   t : number of Julian centuries since J2000.0                      */
//* Return value:                                                       */
//*   the Geometric Mean Anomaly of the Sun in degrees                  */
//***********************************************************************/

function calcGeomMeanAnomalySun(t) {
  var M = 357.52911 + t * (35999.05029 - 0.0001537 * t);
  return M;       // in degrees
}

//***********************************************************************/
//* Name:    calcEccentricityEarthOrbit                                 */
//* Type:    Function                                                   */
//* Purpose: calculate the eccentricity of earth's orbit                */
//* Arguments:                                                          */
//*   t : number of Julian centuries since J2000.0                      */
//* Return value:                                                       */
//*   the unitless eccentricity                                         */
//***********************************************************************/

function calcEccentricityEarthOrbit(t) {
  var e = 0.016708634 - t * (0.000042037 + 0.0000001267 * t);
  return e;       // unitless
}

//***********************************************************************/
//* Name:    calcSunEqOfCenter                                          */
//* Type:    Function                                                   */
//* Purpose: calculate the equation of center for the sun               */
//* Arguments:                                                          */
//*   t : number of Julian centuries since J2000.0                      */
//* Return value:                                                       */
//*   in degrees                                                        */
//***********************************************************************/

function calcSunEqOfCenter(t) {
  var m = calcGeomMeanAnomalySun(t);

  var mrad = degToRad(m);
  var sinm = Math.sin(mrad);
  var sin2m = Math.sin(mrad+mrad);
  var sin3m = Math.sin(mrad+mrad+mrad);

  var C = sinm * (1.914602 - t * (0.004817 + 0.000014 * t)) + sin2m * (0.019993 - 0.000101 * t) + sin3m * 0.000289;
  return C;       // in degrees
}

//***********************************************************************/
//* Name:    calcSunTrueLong                                            */
//* Type:    Function                                                   */
//* Purpose: calculate the true longitude of the sun                    */
//* Arguments:                                                          */
//*   t : number of Julian centuries since J2000.0                      */
//* Return value:                                                       */
//*   sun's true longitude in degrees                                   */
//***********************************************************************/

function calcSunTrueLong(t) {
  var l0 = calcGeomMeanLongSun(t);
  var c = calcSunEqOfCenter(t);

  var O = l0 + c;
  return O;       // in degrees
}

//***********************************************************************/
//* Name:    calcSunApparentLong                                        */
//* Type:    Function                                                   */
//* Purpose: calculate the apparent longitude of the sun                */
//* Arguments:                                                          */
//*   t : number of Julian centuries since J2000.0                      */
//* Return value:                                                       */
//*   sun's apparent longitude in degrees                               */
//***********************************************************************/

function calcSunApparentLong(t) {
  var o = calcSunTrueLong(t);

  var omega = 125.04 - 1934.136 * t;
  var lambda = o - 0.00569 - 0.00478 * Math.sin(degToRad(omega));
  return lambda;      // in degrees
}

//***********************************************************************/
//* Name:    calcMeanObliquityOfEcliptic                                */
//* Type:    Function                                                   */
//* Purpose: calculate the mean obliquity of the ecliptic               */
//* Arguments:                                                          */
//*   t : number of Julian centuries since J2000.0                      */
//* Return value:                                                       */
//*   mean obliquity in degrees                                         */
//***********************************************************************/

function calcMeanObliquityOfEcliptic(t) {
  var seconds = 21.448 - t*(46.8150 + t*(0.00059 - t*(0.001813)));
  var e0 = 23.0 + (26.0 + (seconds/60.0))/60.0;
  return e0;      // in degrees
}

//***********************************************************************/
//* Name:    calcObliquityCorrection                                    */
//* Type:    Function                                                   */
//* Purpose: calculate the corrected obliquity of the ecliptic          */
//* Arguments:                                                          */
//*   t : number of Julian centuries since J2000.0                      */
//* Return value:                                                       */
//*   corrected obliquity in degrees                                    */
//***********************************************************************/

function calcObliquityCorrection(t) {
  var e0 = calcMeanObliquityOfEcliptic(t);

  var omega = 125.04 - 1934.136 * t;
  var e = e0 + 0.00256 * Math.cos(degToRad(omega));
  return e;       // in degrees
}

//***********************************************************************/
//* Name:    calcSunRtAscension                                         */
//* Type:    Function                                                   */
//* Purpose: calculate the right ascension of the sun                   */
//* Arguments:                                                          */
//*   t : number of Julian centuries since J2000.0                      */
//* Return value:                                                       */
//*   sun's right ascension in degrees                                  */
//***********************************************************************/

function calcSunRtAscension(t) {
  var e = calcObliquityCorrection(t);
  var lambda = calcSunApparentLong(t);

  var tananum = (Math.cos(degToRad(e)) * Math.sin(degToRad(lambda)));
  var tanadenom = (Math.cos(degToRad(lambda)));
  var alpha = radToDeg(Math.atan2(tananum, tanadenom));
  return alpha;       // in degrees
}

//***********************************************************************/
//* Name:    calcSunDeclination                                         */
//* Type:    Function                                                   */
//* Purpose: calculate the declination of the sun                       */
//* Arguments:                                                          */
//*   t : number of Julian centuries since J2000.0                      */
//* Return value:                                                       */
//*   sun's declination in degrees                                      */
//***********************************************************************/

function calcSunDeclination(t) {
  var e = calcObliquityCorrection(t);
  var lambda = calcSunApparentLong(t);

  var sint = Math.sin(degToRad(e)) * Math.sin(degToRad(lambda));
  var theta = radToDeg(Math.asin(sint));
  return theta;       // in degrees
}

//***********************************************************************/
//* Name:    calcEquationOfTime                                         */
//* Type:    Function                                                   */
//* Purpose: calculate the difference between true solar time and mean  */
//*     solar time                                                      */
//* Arguments:                                                          */
//*   t : number of Julian centuries since J2000.0                      */
//* Return value:                                                       */
//*   equation of time in minutes of time                               */
//***********************************************************************/

function calcEquationOfTime(t) {
  var epsilon = calcObliquityCorrection(t);
  var l0 = calcGeomMeanLongSun(t);
  var e = calcEccentricityEarthOrbit(t);
  var m = calcGeomMeanAnomalySun(t);

  var y = Math.tan(degToRad(epsilon)/2.0);
  y *= y;

  var sin2l0 = Math.sin(2.0 * degToRad(l0));
  var sinm   = Math.sin(degToRad(m));
  var cos2l0 = Math.cos(2.0 * degToRad(l0));
  var sin4l0 = Math.sin(4.0 * degToRad(l0));
  var sin2m  = Math.sin(2.0 * degToRad(m));

  var Etime = y * sin2l0 - 2.0 * e * sinm + 4.0 * e * y * sinm * cos2l0 - 0.5 * y * y * sin4l0 - 1.25 * e * e * sin2m;

  return radToDeg(Etime)*4.0; // in minutes of time
}

//***********************************************************************/
//* Name:    calcHourAngleSunrise                                       */
//* Type:    Function                                                   */
//* Purpose: calculate the hour angle of the sun at sunrise for the     */
//*         latitude                                                    */
//* Arguments:                                                          */
//*   lat : latitude of observer in degrees                             */
//* solarDec : declination angle of sun in degrees                      */
//* Return value:                                                       */
//*   hour angle of sunrise in radians                                  */
//***********************************************************************/

function calcHourAngleSunrise(lat, solarDec) {
  var latRad = degToRad(lat);
  var sdRad  = degToRad(solarDec);

  var HAarg = (Math.cos(degToRad(90.833))/(Math.cos(latRad)*Math.cos(sdRad))-Math.tan(latRad) * Math.tan(sdRad));

  var HA = (Math.acos(Math.cos(degToRad(90.833))/(Math.cos(latRad)*Math.cos(sdRad))-Math.tan(latRad) * Math.tan(sdRad)));

  return HA;      // in radians
}

//***********************************************************************/
//* Name:    calcHourAngleSunset                                        */
//* Type:    Function                                                   */
//* Purpose: calculate the hour angle of the sun at sunset for the      */
//*         latitude                                                    */
//* Arguments:                                                          */
//*   lat : latitude of observer in degrees                             */
//* solarDec : declination angle of sun in degrees                      */
//* Return value:                                                       */
//*   hour angle of sunset in radians                                   */
//***********************************************************************/

function calcHourAngleSunset(lat, solarDec) {
  var latRad = degToRad(lat);
  var sdRad  = degToRad(solarDec);

  var HAarg = (Math.cos(degToRad(90.833))/(Math.cos(latRad)*Math.cos(sdRad))-Math.tan(latRad) * Math.tan(sdRad));

  var HA = (Math.acos(Math.cos(degToRad(90.833))/(Math.cos(latRad)*Math.cos(sdRad))-Math.tan(latRad) * Math.tan(sdRad)));

  return -HA;     // in radians
}

//***********************************************************************/
//* Name:    calcSunriseUTC                                             */
//* Type:    Function                                                   */
//* Purpose: calculate the Universal Coordinated Time (UTC) of sunrise  */
//*         for the given day at the given location on earth            */
//* Arguments:                                                          */
//*   JD  : julian day                                                  */
//*   latitude : latitude of observer in degrees                        */
//*   longitude : longitude of observer in degrees                      */
//* Return value:                                                       */
//*   time in minutes from zero Z                                       */
//***********************************************************************/

function calcSunriseUTC(JD, latitude, longitude) {
  var t = calcTimeJulianCent(JD);

  // *** Find the time of solar noon at the location, and use
  //     that declination. This is better than start of the 
  //     Julian day
  var noonmin = calcSolNoonUTC(t, longitude);
  var tnoon = calcTimeJulianCent (JD+noonmin/1440.0);

  // *** First pass to approximate sunrise (using solar noon)
  var eqTime = calcEquationOfTime(tnoon);
  var solarDec = calcSunDeclination(tnoon);
  var hourAngle = calcHourAngleSunrise(latitude, solarDec);

  var delta = longitude - radToDeg(hourAngle);
  var timeDiff = 4 * delta;   // in minutes of time
  var timeUTC = 720 + timeDiff - eqTime;  // in minutes

  // *** Second pass includes fractional jday in gamma calc
  var newt = calcTimeJulianCent(calcJDFromJulianCent(t) + timeUTC/1440.0); 
  eqTime = calcEquationOfTime(newt);
  solarDec = calcSunDeclination(newt);
  hourAngle = calcHourAngleSunrise(latitude, solarDec);
  delta = longitude - radToDeg(hourAngle);
  timeDiff = 4 * delta;
  timeUTC = 720 + timeDiff - eqTime; // in minutes

  return timeUTC;
}

//***********************************************************************/
//* Name:    calcSolNoonUTC                                             */
//* Type:    Function                                                   */
//* Purpose: calculate the Universal Coordinated Time (UTC) of solar    */
//*     noon for the given day at the given location on earth           */
//* Arguments:                                                          */
//*   t : number of Julian centuries since J2000.0                      */
//*   longitude : longitude of observer in degrees                      */
//* Return value:                                                       */
//*   time in minutes from zero Z                                       */
//***********************************************************************/

function calcSolNoonUTC(t, longitude) {
  var newt = calcTimeJulianCent(calcJDFromJulianCent(t) + 0.5 + longitude/360.0); 

  var eqTime = calcEquationOfTime(newt);
  var solarNoonDec = calcSunDeclination(newt);
  var solNoonUTC = 720 + (longitude * 4) - eqTime; // min

  return solNoonUTC;
}

//***********************************************************************/
//* Name:    calcSunsetUTC                                              */
//* Type:    Function                                                   */
//* Purpose: calculate the Universal Coordinated Time (UTC) of sunset   */
//*         for the given day at the given location on earth            */
//* Arguments:                                                          */
//*   JD  : julian day                                                  */
//*   latitude : latitude of observer in degrees                        */
//*   longitude : longitude of observer in degrees                      */
//* Return value:                                                       */
//*   time in minutes from zero Z                                       */
//***********************************************************************/

function calcSunsetUTC(JD, latitude, longitude) {
  var t = calcTimeJulianCent(JD);

  // *** Find the time of solar noon at the location, and use
  //     that declination. This is better than start of the 
  //     Julian day
  var noonmin = calcSolNoonUTC(t, longitude);
  var tnoon = calcTimeJulianCent (JD+noonmin/1440.0);

  // First calculates sunrise and approx length of day
  var eqTime = calcEquationOfTime(tnoon);
  var solarDec = calcSunDeclination(tnoon);
  var hourAngle = calcHourAngleSunset(latitude, solarDec);

  var delta = longitude - radToDeg(hourAngle);
  var timeDiff = 4 * delta;
  var timeUTC = 720 + timeDiff - eqTime;

  // first pass used to include fractional day in gamma calc
  var newt = calcTimeJulianCent(calcJDFromJulianCent(t) + timeUTC/1440.0); 
  eqTime = calcEquationOfTime(newt);
  solarDec = calcSunDeclination(newt);
  hourAngle = calcHourAngleSunset(latitude, solarDec);

  delta = longitude - radToDeg(hourAngle);
  timeDiff = 4 * delta;
  timeUTC = 720 + timeDiff - eqTime; // in minutes

  return timeUTC;
}

// Returns the decimal latitude from the degrees, minutes and seconds entered 
// into the form    
function getLatitude(latDeg, latMin, latSec) {
  var degs = latDeg;
  var mins = latMin;
  var secs = latSec;

  var decLat;
  if (degs >= 0) {
    decLat = degs + (mins / 60) + (secs / 3600);
  } 
  else if (degs < 0) {
    decLat = degs - (mins / 60) - (secs / 3600);
  } 
  else {
    return -9999;
  }
  return decLat;
}   

// Returns the decimal longitude from the degrees, minutes and seconds entered 
// into the form    
function getLongitude(lonDeg, lonMin, lonSec) {
  var degs = lonDeg;
  var mins = lonMin;
  var secs = lonSec;

  var decLon;
  if (degs >= 0) {
    decLon = degs + (mins / 60) + (secs / 3600);
  } 
  else if (degs < 0) {
    decLon = degs - (mins / 60) - (secs / 3600);
  } 
  else {
    return -9999;
  }
  return decLon;
}

//***********************************************************************/
//* Name:    findRecentSunrise                                          */
//* Type:    Function                                                   */
//* Purpose: calculate the julian day of the most recent sunrise        */
//*     starting from the given day at the given location on earth      */
//* Arguments:                                                          */
//*   JD  : julian day                                                  */
//*   latitude : latitude of observer in degrees                        */
//*   longitude : longitude of observer in degrees                      */
//* Return value:                                                       */
//*   julian day of the most recent sunrise                             */
//***********************************************************************/

function findRecentSunrise(jd, latitude, longitude) {
  var julianday = jd;

  var time = calcSunriseUTC(julianday, latitude, longitude);
  while (!isNumber(time)) {
    julianday -= 1.0;
    time = calcSunriseUTC(julianday, latitude, longitude);
  }

  return julianday;
}

//***********************************************************************/
//* Name:    findRecentSunset                                           */
//* Type:    Function                                                   */
//* Purpose: calculate the julian day of the most recent sunset         */
//*     starting from the given day at the given location on earth      */
//* Arguments:                                                          */
//*   JD  : julian day                                                  */
//*   latitude : latitude of observer in degrees                        */
//*   longitude : longitude of observer in degrees                      */
//* Return value:                                                       */
//*   julian day of the most recent sunset                              */
//***********************************************************************/

function findRecentSunset(jd, latitude, longitude) {
  var julianday = jd;

  var time = calcSunsetUTC(julianday, latitude, longitude);
  while (!isNumber(time)) {
    julianday -= 1.0;
    time = calcSunsetUTC(julianday, latitude, longitude);
  }

  return julianday;
}

//***********************************************************************/
//* Name:    findNextSunrise                                            */
//* Type:    Function                                                   */
//* Purpose: calculate the julian day of the next sunrise               */
//*     starting from the given day at the given location on earth      */
//* Arguments:                                                          */
//*   JD  : julian day                                                  */
//*   latitude : latitude of observer in degrees                        */
//*   longitude : longitude of observer in degrees                      */
//* Return value:                                                       */
//*   julian day of the next sunrise                                    */
//***********************************************************************/

function findNextSunrise(jd, latitude, longitude) {
  var julianday = jd;

  var time = calcSunriseUTC(julianday, latitude, longitude);
  while (!isNumber(time)) {
    julianday += 1.0;
    time = calcSunriseUTC(julianday, latitude, longitude);
  }

  return julianday;
}

//***********************************************************************/
//* Name:    findNextSunset                                             */
//* Type:    Function                                                   */
//* Purpose: calculate the julian day of the next sunset                */
//*     starting from the given day at the given location on earth      */
//* Arguments:                                                          */
//*   JD  : julian day                                                  */
//*   latitude : latitude of observer in degrees                        */
//*   longitude : longitude of observer in degrees                      */
//* Return value:                                                       */
//*   julian day of the next sunset                                     */
//***********************************************************************/

function findNextSunset(jd, latitude, longitude) {
  var julianday = jd;

  var time = calcSunsetUTC(julianday, latitude, longitude);
  while (!isNumber(time)) {
    julianday += 1.0;
    time = calcSunsetUTC(julianday, latitude, longitude);
  }

  return julianday;
}

//***********************************************************************/
//* Name:    timeString                                                 */
//* Type:    Function                                                   */
//* Purpose: convert time of day in minutes to a zero-padded string     */
//*     suitable for printing to the form text fields                   */
//* Arguments:                                                          */
//*   minutes : time of day in minutes                                  */
//* Return value:                                                       */
//*   string of the format HH:MM:SS, minutes and seconds are zero padded*/
//***********************************************************************/

function timeString(minutes) {
  var floatHour = minutes / 60;
  var hour = Math.floor(floatHour);
  var floatMinute = 60 * (floatHour - Math.floor(floatHour));
  var minute = Math.floor(floatMinute);
  var floatSec = 60 * (floatMinute - Math.floor(floatMinute));
  var second = Math.floor(floatSec);

  var timeStr = hour + ":";
  if (minute < 10)    //  i.e. only one digit
  timeStr += "0" + minute + ":";
  else
  timeStr += minute + ":";
  if (second < 10)    //  i.e. only one digit
  timeStr += "0" + second;
  else
  timeStr += second;

  return timeStr;
}

//**************************************************************************/
//* Name:    SunriseSunset                                                 */
//* Type:    Main Function called by form controls                         */
//* Purpose: calculate time of sunrise and sunset for the entered date     */
//*     and location.  In the special cases near earth's poles,            */
//*     the date of nearest sunrise and set are reported.                  */
//* Arguments:                                                             */
//*   month, day, year, zone,                                              */
//*   lonDegree, lonMinutes, lonSeconds, latDegree, latMinutes, latSeconds */
//* Return value:                                                          */
//*   ojbect containing riseHours, riseMinutes, setHours, setMinutes, noon,*/ 
//*   declination, minutes                                                 */
//**************************************************************************/

function SunriseSunset(month, day, year, zone, lonDeg, lonMin, lonSec, latDeg, latMin, latSec) {

  var riseHours, riseMinutes, setHours, setMinutes, noon, declination, minutes;

  var latitude = getLatitude(latDeg, latMin, latSec);
  var longitude = getLongitude(lonDeg, lonMin, lonSec);

  if((latitude >= -90) && (latitude < -89.8)) {
    latDeg = -89.8;
    latitude = -89.8;
  }
  if ((latitude <= 90) && (latitude > 89.8)) {
    latDeg = 89.8;
    latitude = 89.8;
  }

  var JD = (calcJD(year, month + 1, day));
  var doy = calcDayOfYear(month + 1, day, isLeapYear(year));
  var T = calcTimeJulianCent(JD);
  var alpha = calcSunRtAscension(T);
  var theta = calcSunDeclination(T);
  var Etime = calcEquationOfTime(T);

  var eqTime = Etime;
  var solarDec = theta;

  minutes = (Math.floor(100*eqTime))/100;
  declination = (Math.floor(100*(solarDec)))/100;

  // Calculate sunrise for this date
  // if no sunrise is found, set flag nosunrise
  var nosunrise = false;

  var riseTimeGMT = calcSunriseUTC(JD, latitude, longitude);
  if (!isNumber(riseTimeGMT)) {
    nosunrise = true;
  }

  // Calculate sunset for this date
  // if no sunset is found, set flag nosunset
  var nosunset = false;
  var setTimeGMT = calcSunsetUTC(JD, latitude, longitude);
  if (!isNumber(setTimeGMT)) {
    nosunset = true;
  }

  var daySavings = 0;
  var riseTimeLST, setTimeLST;
  if (!nosunrise) { // Sunrise was found
    riseTimeLST = riseTimeGMT - (60 * zone) + daySavings;   
    riseHours = Math.floor(riseTimeLST/60); //  in minutes
    riseMinutes = Math.floor(riseTimeLST - 60*riseHours + 0.5);
    if (riseMinutes > 59) {
      riseMinutes = 0;
      riseHours++;
    }
  }

  if (!nosunset) { // Sunset was found
    setTimeLST = setTimeGMT - (60 * zone) + daySavings;
    setHours = Math.floor(setTimeLST/60);
    setMinutes = Math.floor(setTimeLST - 60*setHours + 0.5);
    if (setMinutes > 59) {
      setMinutes = 0;
      setHours++;
    }
  }

  // Calculate solar noon for this date
  var solNoonGMT = calcSolNoonUTC(T, longitude);
  var solNoonLST = solNoonGMT - (60 * zone) + daySavings;

  var solnStr = timeString(solNoonLST);
  var utcSolnStr = timeString(solNoonGMT);

  noon = solnStr;

  //***********Convert lat and long to standard format
  convLatLong(lonDeg, lonMin, lonSec, latDeg, latMin, latSec);

  // report special cases of no sunrise
  if (nosunrise) { 
    // if Northern hemisphere and spring or summer, OR  
    // if Southern hemisphere and fall or winter, use 
    // previous sunrise and next sunset
    if ( ((latitude > 66.4) && (doy > 79) && (doy < 267)) || ((latitude < -66.4) && ((doy < 83) || (doy > 263))) ) {
      newjd = findRecentSunrise(JD, latitude, longitude);
      newtime = calcSunriseUTC(newjd, latitude, longitude) - (60 * zone) + daySavings;
      if (newtime > 1440) {
        newtime -= 1440;
        newjd += 1.0;
      }
      if (newtime < 0) {
        newtime += 1440;
        newjd -= 1.0;
      }

      riseHours = Math.floor(newtime/60);
      riseMinutes = Math.floor(riseTimeLST - 60*riseHours + 0.5);
    }

    // if Northern hemisphere and fall or winter, OR 
    // if Southern hemisphere and spring or summer, use 
    // next sunrise and previous sunset
    else if ( ((latitude > 66.4) && ((doy < 83) || (doy > 263))) || ((latitude < -66.4) && (doy > 79) && (doy < 267)) ) {
      newjd = findNextSunrise(JD, latitude, longitude);
      newtime = calcSunriseUTC(newjd, latitude, longitude) - (60 * zone) + daySavings;
      if (newtime > 1440) {
        newtime -= 1440;
        newjd += 1.0;
      }
      if (newtime < 0) {
        newtime += 1440;
        newjd -= 1.0;
      }
      riseHours = Math.floor(newtime/60);
      riseMinutes = Math.floor(riseTimeLST - 60*riseHours + 0.5);
    }
  }

  if (nosunset) { 
    // if Northern hemisphere and spring or summer, OR
    // if Southern hemisphere and fall or winter, use 
    // previous sunrise and next sunset
    if ( ((latitude > 66.4) && (doy > 79) && (doy < 267)) || ((latitude < -66.4) && ((doy < 83) || (doy > 263))) ) {
      newjd = findNextSunset(JD, latitude, longitude);
      newtime = calcSunsetUTC(newjd, latitude, longitude) - (60 * zone) + daySavings;
      if (newtime > 1440) {
        newtime -= 1440;
        newjd += 1.0;
      }
      if (newtime < 0) {
        newtime += 1440;
        newjd -= 1.0;
      }
      setHours = Math.floor(newtime/60);
      setMinutes = Math.floor(setTimeLST - 60*setHours + 0.5);
    }

    // if Northern hemisphere and fall or winter, OR
    // if Southern hemisphere and spring or summer, use 
    // next sunrise and last sunset
    else if ( ((latitude > 66.4) && ((doy < 83) || (doy > 263))) || ((latitude < -66.4) && (doy > 79) && (doy < 267)) ) {
      newjd = findRecentSunset(JD, latitude, longitude);
      newtime = calcSunsetUTC(newjd, latitude, longitude) - (60 * zone) + daySavings;
      if (newtime > 1440) {
        newtime -= 1440;
        newjd += 1.0;
      }
      if (newtime < 0) {
        newtime += 1440;
        newjd -= 1.0;
      }
      setHours = Math.floor(newtime/60);
      setMinutes = Math.floor(setTimeLST - 60*setHours + 0.5);
      noon = "N/A";
    }
  }
  return {
    riseHours:riseHours, riseMinutes:riseMinutes, setHours:setHours, setMinutes:setMinutes,
    noon:noon, declination:declination, minutes:minutes,
    nosunrise:nosunrise, nosunset:nosunset
  };
}