// Program version
var versionNum = '4.0.8';
var isTiger = false; //for Mac OS X Tiger support

// Default values:
var defaultPoint = '1159068000'; // 서울 동작구 신대방2동 - 기상청
var defaultPoint2nd = '2611053000'; // 부산 중구 대청동
var currentPoint, currentPointIndex;
var defaultIconIndex = 8;
var defaultWindIconIndex = 0;
var isFahrenheit = false;
var snowFallingEffect = false;
var isSnowing = false;
var defaultSkin = 'wood1';

// Real values:
var myPointSaved, myPoint2ndSaved;
var myPoint, myPoint2nd;
var stationAndLocationCode, stationAndLocationCode2nd;
var actualIconIndex = defaultIconIndex;
var actualWindIconIndex = defaultWindIconIndex;
var actualFahrenheit = isFahrenheit;
var actualSnowFallingEffect = snowFallingEffect;
var skin = defaultSkin;

// URLs for weather updates
var currentWeatherURL = 'http://www.kma.go.kr/wid/queryCurrentFullWeather.jsp?stnId=';
var weatherDataURL = 'http://www.kma.go.kr/cgi-bin/aws/nph-aws_txt_min?';
var shortTermForecastURL = 'http://www.kma.go.kr/wid/queryDFS.jsp?';
var midTermForecastURL = 'http://www.kma.go.kr/weather/forecast/mid-term-xml.jsp?stnId=108';

var weatherRequest; // for weather updates
var weatherDataRequest; // for weather data updates
var shortTermForecastRequest, midTermForecastRequest, midTermForecastLastUpdated; // for weather forecast updates
var versionRequest; // for version checks

var today;
var timer = null; // for data fetching animation
var iconName;
var weatherIconNames = { 0: 'Simple', 1: 'Shiny', 2: 'Samurize', 3: 'Kapsules', 4: 'Konfabulator', 
5: 'Jivesucker', 6: '7even', 7: 'Babasse', 8: 'Epona', 9: 'Apple', 10: 'Sticker', 11: 'Tick', 12: 'Plain', noOfIcons: 13 };
var windIcon = [{name: 'vane', width: 18, height: 48}, {name: 'arrow', width: 25, height: 25}];

var weather; // where the current weather condition stored
var humidity;
var timeElement;
var weatherElement; // forecast weather condition stored here
var weatherData = []; // where all the other weather data stored
var todayLength; // the number of today's forecast columns
var sunDataPt1 = [], sunDataPt2 = []; // where the current sunrise and sunset time info stored
var sun_or_moon;

//==========================================================
//  Constants that control wind vane "sproing" animation 
//  - the idea is borrowed from Apple's World Clock widget
//==========================================================
//  To simulate a mechanical wind vane motion, we will:
//    - Go to T+n
//    - Go to T-n
//    - Go to T+(n/2)
//    - Go to T-(n/2)
//    - Go to T+(n/4)
//    - Go to T-(n/4)
//    - Go to T

var sweepbackAnimateInterval = 35;
var sweepbackMax = 30; // in degrees out of 180
var sweepbackAmounts = [-2*sweepbackMax, 1.5*sweepbackMax,
										 -sweepbackMax, 0.75*sweepbackMax,
										 -0.5*sweepbackMax, 0.25*sweepbackMax];

var sweepbackAnimation = {step:0, timer:null};
var originalAngle = 0; // angle of the wind vane
var windDirection, windForce;

// Regular Expressions for data extracting
var updateWeatherDataRegEx = /<td>(<font color=(blue|red)>[^"\r\n]*<\/font>|\.)<\/td>(<td>[^"\r\n]*<\/td>|<td class=textb>[^"\r\n]*<\/td>)(<td>[^"\r\n]*<\/td>|<td class=textb>[^"\r\n]*<\/td>)(<td>[^"\r\n]*<\/td>|<td class=textb>[^"\r\n]*<\/td>)(<td>[^"\r\n]*<\/td>|<td class=textb>[^"\r\n]*<\/td>)<td>[^"\r\n]*<\/td><td class=textg>[^"\r\n]*<\/td><td class=textg>[^"\r\n]*<\/td><td class=textg>[^"\r\n]*<\/td><td class=textb>[^"\r\n]*<\/td><td class=textb>[^"\r\n]*<\/td><td class=textb>[^"\r\n]*<\/td>(<td>[^"\r\n]*<\/td>)?/i;
var updateWeatherDataRegEx2 = /(?:<td>)|(?:<\/td><td>)|(?:<\/td><td class=textg>)|(?:<\/td><td class=textb>)|(?:<\/td><td>)|(?:<\/td>)/g;

// for the localization
function getLocalizedString(string) {
	try { string = localizedStrings[string] || string; } catch (e) {}
	return string;
}

function getLocalizedProvinceName(key) {
	try {
		return localizedProvinceNames[key];
	} catch (ex) {}

	return key;
}

function getLocalizedAreaName(key) {
	try {
		return localizedAreaNames[key];
	} catch (ex) {}

	return key;
}

function getLocalizedDayOfWeekName(key) {
	return localizedDayOfWeekNames[key%7];
}

// Create unique key for each instance of widget
function createkey(key) {
  return widget.identifier + '-' + key;
}

function addClass(elm, className) {
  var currentClasses = elm.className.split(' ');
  currentClasses.push(className);
  elm.className = currentClasses.join(' ');
}

function setup() {
  if ((navigator.appVersion.indexOf('AppleWebKit/4') > 0) || (navigator.appVersion.indexOf('AppleWebKit/3') > 0) || (navigator.appVersion.indexOf('AppleWebKit/2') > 0)) {
    isTiger = true;
  }

  var doc = document;
  if (isTiger) {
    doc.getElementById('tigerface').style.display = 'block';
    var borderDiv = doc.getElementById('widget-border');
    addClass(borderDiv, 'tiger');
  }

  var doneString = getLocalizedString('Done');
  CreateGlassButton('done', { text: doneString, onclick: 'hidePrefs' });
  CreateInfoButton('info', { 'backgroundStyle': 'black', 'creationFunction': 'CreateInfoButton', 'foregroundStyle': 'white', 'frontID': 'front', 'onclick': 'showPrefs' });

  // Try to retrieve preferences, if they exist
  if (window.widget) {
    myPointSaved = widget.preferenceForKey(createkey('point'));
    myPoint2ndSaved = widget.preferenceForKey(createkey('point2nd'));
    currentPointIndex = widget.preferenceForKey(createkey('pointindex'));
    actualIconIndex = widget.preferenceForKey(createkey('iconindex'));
    actualWindIconIndex = widget.preferenceForKey(createkey('windiconindex'));
    actualFahrenheit = widget.preferenceForKey(createkey('fahrenheit'));
    actualSnowFallingEffect = widget.preferenceForKey(createkey('snoweffect'));
    skin = widget.preferenceForKey(createkey('skinkey'));
  }

  if (myPointSaved === undefined) {
    myPoint = new Point(defaultPoint);
    widget.setPreferenceForKey(myPoint.z3, createkey('point'));
    stationAndLocationCode = getStationAndLocationCode(myPoint);
  } else {
    myPoint = new Point(myPointSaved);
    stationAndLocationCode = getStationAndLocationCode(myPoint);
  }

  if (myPoint2ndSaved === undefined) {
    myPoint2nd = new Point(defaultPoint2nd);
    widget.setPreferenceForKey(myPoint2nd.z3, createkey('point2nd'));
    stationAndLocationCode2nd = getStationAndLocationCode(myPoint2nd);
  } else {
    myPoint2nd = new Point(myPoint2ndSaved);
    stationAndLocationCode2nd = getStationAndLocationCode(myPoint2nd);
  }

  if (currentPointIndex === undefined) {
    currentPointIndex = 0;
    widget.setPreferenceForKey(currentPointIndex, createkey('pointindex'));
  }

  currentPoint = (currentPointIndex === 0) ? myPoint : myPoint2nd;

  populateAreaNamePopup();

  populateProvinceSelect(currentPoint);

  if (actualFahrenheit === undefined) {
    actualFahrenheit = isFahrenheit;
    widget.setPreferenceForKey(actualFahrenheit, createkey('fahrenheit'));
  }

  populateTempUnits();

  if (actualIconIndex === undefined) {
    actualIconIndex = defaultIconIndex;
    widget.setPreferenceForKey(actualIconIndex, createkey('iconindex'));
  }

  populateIconSelect();
  doc.getElementById('icon-popup').options[actualIconIndex].selected = true;

  if (actualWindIconIndex === undefined) {
    actualWindIconIndex = defaultWindIconIndex;
    widget.setPreferenceForKey(actualWindIconIndex, createkey('windiconindex'));
  }

  if (actualSnowFallingEffect === undefined) {
    actualSnowFallingEffect = snowFallingEffect;
    widget.setPreferenceForKey(actualSnowFallingEffect, createkey('snoweffect'));
  }

  if (skin === undefined) {
    skin = defaultSkin;
    widget.setPreferenceForKey(skin, createkey('skinkey'));
  }
  setSkin(skin);

  doc.getElementById('humidity').title = getLocalizedString('Humidity');
  doc.getElementById('visibility').title = getLocalizedString('Visibility');
  doc.getElementById('paper-plane').title = getLocalizedString('FeelsLike');

  iconName = weatherIconNames[actualIconIndex];

  // Load values into form
  doc.getElementById('province-label').innerText =  getLocalizedString('Province');
  doc.getElementById('city-label').innerText =  getLocalizedString('City');
  doc.getElementById('town-label').innerText =  getLocalizedString('Town');
  doc.getElementById('unit-label').innerText =  getLocalizedString('Degrees');
  doc.getElementById('icon-label').innerText =  getLocalizedString('Icon');
  doc.getElementById('snow-label').innerText =  getLocalizedString('SnowEffect');
  doc.getElementById('border-label').innerText =  getLocalizedString('BorderSelect');
  doc.getElementById('version-label').innerText =  getLocalizedString('Version') + versionNum;

  getWeather();
  checkForNewVersion();
}

function getWeather() {
	if (weatherRequest) weatherRequest.abort();
	
	today = new Date();
	var localGMTOffset = today.getTimezoneOffset();    
	var targetGMTOffset = 60 * 9;    // Korean time zone is GMT+0900
	today.setTime(today.getTime() + ((localGMTOffset + targetGMTOffset) * 60 * 1000));
	
	var stn_id;
  if (currentPoint == myPoint) {
    stn_id = stationAndLocationCode.station;
  } else {
    stn_id = stationAndLocationCode2nd.station;
  }
	
	var currentWeatherRequestURL = currentWeatherURL + stn_id;
	
	weatherData = [];
	
	weatherRequest = new XMLHttpRequest();
	weatherRequest.onreadystatechange = updateWeather;
	if (!isTiger) {
		weatherRequest.overrideMimeType('text/xml');
	}
	weatherRequest.open('GET', currentWeatherRequestURL, true);
	weatherRequest.send(null);
	
	if (isSnowing) {
		snowStorm.toggleSnow();
		isSnowing = false;
	}
	
	endRainAlertAnimation();
	startFetchingAnimation();
	getShortTermForecast();
  if (!isTiger) {
    // weekly forecast updates every 6 and 18 hours
    if ((midTermForecastLastUpdated === undefined) || (today.getDate() !== midTermForecastLastUpdated.getDate()) || ((today.getHours() >= 6) && (midTermForecastLastUpdated.getHours() < 6)) || ((today.getHours() >= 18) && (midTermForecastLastUpdated.getHours() < 18))) {
      getMidTermForecast();
    }
  } 
}

function updateWeather() {
  try {
    if (weatherRequest.readyState == 4 && weatherRequest.status == 200) { // OK everything
      var response, result, visibility;
      if (!isTiger) {
        response = weatherRequest.responseXML;
        result = response.evaluate('/wid/body/data/ww|/wid/body/data/wwKo|/wid/body/data/vs|/wid/body/data/hm', response, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
        if (result.snapshotLength !== 0) {
          var element = [];
          for (var i=0, len=result.snapshotLength; i < len; i++) {
            element[i] = result.snapshotItem(i);
          }

          if (element[0].textContent !== '') {
            weather = element[0].textContent.toLowerCase();
          } else {
            weather = translateKoToEn(element[1].textContent);
          }

          visibility = element[2].textContent;
          humidity = element[3].textContent;
        }
      } else {
        response = weatherRequest.responseText;				
        result = getDOMfromXML(response);
        var nodes = result.getElementsByTagName('ww');
        if (nodes[0].firstChild !== null) {
          weather = nodes[0].firstChild.data.toLowerCase();
        } else {
          nodes = result.getElementsByTagName('wwKo');
          if (nodes[0].firstChild !== null) {
            weather = translateKoToEn(nodes[0].firstChild.data);
          }
        }
        nodes = result.getElementsByTagName('vs');
        if (nodes[0].firstChild !== null) {
          visibility = nodes[0].firstChild.data;
        }
        nodes = result.getElementsByTagName('hm');
        if (nodes[0].firstChild !== null) {
          humidity = nodes[0].firstChild.data;
        }
      }
      weatherRequest = null;
      
      if (!weather) {
        var stn_id;
        if (currentPoint == myPoint) {
          stn_id = stationAndLocationCode.nearCityStation;
        } else {
          stn_id = stationAndLocationCode2nd.nearCityStation;
        }
        if (stn_id) {
          getNearCityWeatherCondition(stn_id);
        }
      } else {
        document.getElementById('weatherStationName').style.display = 'none';
      }

      var doc = document;
      if ((visibility !== undefined) && (visibility !== '')) {
        visibility = parseFloat(visibility);
        if (visibility >= 100) {
          visibility = visibility / 100;
        }
        doc.getElementById('visibility').innerText = visibility + 'km';
      } else {
        doc.getElementById('visibility').innerText = '- km';
      }

      var sunData = (currentPoint == myPoint) ? sunDataPt1 : sunDataPt2;

      if (sunData.timeTag !== today.toDateString()+currentPoint.z3) { // If the stored data are out of date
        getSunTime();
        updateMoonPhases();
      } else {
        document.getElementById('risenfall').innerHTML = '&uarr;' + sunData.riseHours + ':' + sunData.riseMinutes + ' &darr;' + sunData.setHours + ':' + sunData.setMinutes;
      }

      sun_or_moon = getDayNight();
      
      if (!weather) {
        setTimeout(getWeatherData, 200);
      } else {
        getWeatherData(); // retrieving temperature, humidity, wind direction and wind speed data
      }
      
      endFetchingAnimation();

      // It's not raining or the raining indicator data are not available
      if (((weatherData[1]) && (!weatherData[1].match(/blue/))) || (weatherData[1] === undefined)) {
        doc.getElementById('board').className = sun_or_moon;

        if (isTiger) {
          doc.getElementById('area-name').style.backgroundImage = 'url(images/popup/area-name-popup-' +  sun_or_moon + '.png)';
        } else {
          doc.getElementById('area-name').style.backgroundImage = 'url(images/popup/area-name-popup_left-' + sun_or_moon + '.png), url(images/popup/area-name-popup_middle-' + sun_or_moon + '.png), url(images/popup/area-name-popup_right-' + sun_or_moon + '.png)';
        }
      }

      var townSelect = doc.getElementById('town-popup');
      var townString = townSelect.options[townSelect.selectedIndex].text;
      if (townString.length > 14) {
        townString = townString.substr(0, 12) + '…';
      }
      doc.getElementById('area-name-label').innerText = townString;
    }
  }
  catch(exception) {
    drawCheckFailure(getLocalizedString('Failed') + exception);
  }
}

function getNearCityWeatherCondition(nearCityStation) {
	var currentWeatherRequestURL = currentWeatherURL + nearCityStation;
	
	weatherRequest = new XMLHttpRequest();
	weatherRequest.onreadystatechange = updateNearCityWeatherCondition;
	if (!isTiger) {
		weatherRequest.overrideMimeType('text/xml');
	}
	weatherRequest.open('GET', currentWeatherRequestURL, true);
	weatherRequest.send(null);
}

function updateNearCityWeatherCondition() {
  try {
    if (weatherRequest.readyState == 4 && weatherRequest.status == 200) { // OK everything
      var response, result, weatherStationNameDiv;
      if (!isTiger) {
        response = weatherRequest.responseXML;
        result = response.evaluate('/wid/body/data/stnKo|/wid/body/data/stnEn|/wid/body/data/ww|/wid/body/data/wwKo', response, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
        if (result.snapshotLength !== 0) {
          var element = [];
          for (var i=0, len=result.snapshotLength; i < len; i++) {
            element[i] = result.snapshotItem(i);
          }
          
          weatherStationNameDiv = document.getElementById('weatherStationName');
          if (document.getElementById('humidity').title !== 'Humidity') { // current locale is Korean
            weatherStationNameDiv.innerText = element[0].textContent;
          } else {
            weatherStationNameDiv.innerText = element[1].textContent;
          }
          weatherStationNameDiv.style.display = 'block';
          
          if (element[2].textContent !== '') {
            weather = element[2].textContent.toLowerCase();
          } else {
            weather = translateKoToEn(element[3].textContent);
          }
        }
      } else {
        response = weatherRequest.responseText;				
        result = getDOMfromXML(response);
        
        weatherStationNameDiv = document.getElementById('weatherStationName');
        if (document.getElementById('humidity') !== 'Humidity') {
          weatherStationNameDiv.innerText = result.getElementsByTagName('stnKo')[0].firstChild.data;
        } else {
          weatherStationNameDiv.innerText = result.getElementsByTagName('stnEn')[0].firstChild.data;
        }
        weatherStationNameDiv.style.display = 'block';
        
        var nodes = result.getElementsByTagName('ww');
        if (nodes[0].firstChild !== null) {
          weather = nodes[0].firstChild.data.toLowerCase();
        } else {
          nodes = result.getElementsByTagName('wwKo');
          if (nodes[0].firstChild !== null) {
            weather = translateKoToEn(nodes[0].firstChild.data);
          }
        }
      }
      weatherRequest = null;
    }
  }
  catch(exception) {
    drawCheckFailure(getLocalizedString('Failed') + exception);
  }
}

function getDOMfromXML(xml) {
  var d = document.createElement('div');
  xml = xml.substring(xml.indexOf('<body') + xml.substring(xml.indexOf('<body')).indexOf('>')+1);
  xml = xml.substring(0, xml.indexOf('</body>'));
  d.innerHTML = xml;
  return d;
}

function translateKoToEn(word) {
	switch (word) {
		case '맑음':
			return 'sunny';
		case '구름조금':
			return 'partly cloudy';
		case '구름많음':
			return 'mostly cloudy';
		case '흐림':
			return 'cloudy';
		case '소나기':
			return 'rain shower';
		case '비':
		case '흐리고 비':
		case '구름많고 비':
			return 'rain';
		case '약한비계속':
			return 'drizzle';
		case '눈':
		case '구름많고 눈':
			return 'snow';
		case '비 또는 눈':
		case '눈 또는 비':
		case '흐리고 비 또는 눈':
		case '흐리고 눈 또는 비':
		case '구름많고 비 또는 눈':
		case '구름많고 눈 또는 비':
			return 'rain and snow';
		case '천둥번개':
			return 'thunderstorm';
		case '안개':
			return 'fog';
		case '박무':
			return 'mist';
		case '연무':
			return 'haze';
		case '황사':
			return 'dust';
		default:
			return '';
	}
}

function showAndStopSnowFalling() {
	switch (weather) {
		//case 'rain/snow':
		case 'rain':
		case 'rain shower':
		case 'drizzle':
		case 'drizzle freezing':
		case 'thunderstorm/rain':
		case 'thunderstorm':
			if (actualSnowFallingEffect && isSnowing) {
				snowStorm.toggleSnow();
				isSnowing = false;
			}
			var doc = document;
			doc.getElementById('rain-probability').style.opacity = '1.0';
			doc.getElementById('board').className = 'raining';
			doc.getElementById('area-name').style.opacity = 0.8;
			doc.getElementById('stars').style.display = 'none';
			break;
		case 'snow':
		case 'snow grains':
		case 'snow shower':
		case 'blowing snow':
			if (actualSnowFallingEffect && !isSnowing) {
				snowStorm.randomizeWind();
				snowStorm.toggleSnow();
				isSnowing = true;
			}
			document.getElementById('area-name').style.opacity = 1.0;
			break;
		default:
			if (actualSnowFallingEffect && !isSnowing && parseFloat(weatherData[7]) <= 1.0) {
				snowStorm.randomizeWind();
				snowStorm.toggleSnow();
				isSnowing = true;
				document.getElementById('area-name').style.opacity = 1.0;
			}
	}
}

function showWeatherIcon() {
	if ((weather !== '') && (weather !== undefined)) { // isn't it empty?
	  var temperature = parseFloat(weatherData[7]);
	  var windSpeed = parseFloat(weatherData[10]);
		switch (weather) {
			case 'sunny':
				if ((sun_or_moon == 'sun') && (temperature >= 30)) { // degrees over 30 Celsius
					document.getElementById('weathericon').src = 'images/weather_icons/' + iconName + '/sunny-sun-hot.png';
				} else if (sun_or_moon == 'moon') {
					document.getElementById('weathericon').src = 'images/weather_icons/' +iconName + '/sunny-moon.png';
					if (!isTiger) {
						var stars = document.getElementById('stars');
						for (var i = stars.children.length - 1; i >= 0; i--) {
							stars.children[i].style.top = (Math.random() * 80 + 20) + 'px';
							stars.children[i].style.left = (Math.random() * 260 + 26) + 'px';
						}
						stars.style.display = 'block';
					}
				} else {
					document.getElementById('weathericon').src = 'images/weather_icons/' + iconName + '/sunny-sun.png';
				}
				break;
			case 'partly cloudy':
			case 'funnel cloud':
				document.getElementById('weathericon').src = 'images/weather_icons/' + iconName + '/' + weather + '-' + sun_or_moon + '.png';
				break;
			case 'most cloudy':
			case 'mostly cloudy':
				document.getElementById('weathericon').src = 'images/weather_icons/' + iconName + '/mostly cloudy-' + sun_or_moon + '.png';
				break;
			case 'preceding rain':
			case 'preceding drizzle':
			case 'preceding shower snow':
			case 'preceding shower rain':
			case 'preceding rain/snow':
			case 'preceding snow':
			case 'preceding thunderstorm':
			case 'precipitation within sight':
				document.getElementById('weathericon').src = 'images/weather_icons/' + iconName + '/cloudy.png';
				break;
			case 'fog/ice fog':
				if (iconName == 'Tick') {
					document.getElementById('weathericon').src = 'images/weather_icons/' + iconName + '/fog-' + sun_or_moon + '.png';
				} else {
					document.getElementById('weathericon').src = 'images/weather_icons/' + iconName + '/fog.png';
				}
				break;
			case 'mist':
			case 'haze':
			case 'preceding fog':
				if ((iconName == 'Tick') || (iconName == 'Plain')) {
					document.getElementById('weathericon').src = 'images/weather_icons/' + iconName + '/mist-' + sun_or_moon + '.png';
				} else {
					document.getElementById('weathericon').src = 'images/weather_icons/' + iconName + '/mist.png';
				}
				break;
			case 'rain/snow':
			case 'rain and snow':
				if (temperature < 0) { // degrees under 0 Celsius
					document.getElementById('weathericon').src = 'images/weather_icons/' + iconName + '/rain-snow-icy.png';
				} else {
					document.getElementById('weathericon').src = 'images/weather_icons/' + iconName + '/rain-snow.png';
				}
				break;
			case 'thunderstorm/rain':
				document.getElementById('weathericon').src = 'images/weather_icons/' + iconName + '/thunderstorm-rain.png';
				break;
			case 'lightning':
				document.getElementById('weathericon').src = 'images/weather_icons/' + iconName + '/thunderstorm.png';
				break;	
			case 'rain':
				if ((windSpeed >= 10) && (temperature >= 0)) { // wind speed over 10 m/s and degrees over 0 Celsius
					document.getElementById('weathericon').src = 'images/weather_icons/' + iconName + '/rain-windy.png';
				} else if (temperature < 0) { // degrees under 0 Celsius
					document.getElementById('weathericon').src = 'images/weather_icons/' + iconName + '/rain-icy.png';
				} else {
					document.getElementById('weathericon').src = 'images/weather_icons/' + iconName + '/rain.png';
				}
				break;
			case 'cloudy':
				if (windSpeed >= 10) { // wind speed over 10 m/s
					document.getElementById('weathericon').src = 'images/weather_icons/' + iconName + '/cloudy-windy.png';
				} else {
					document.getElementById('weathericon').src = 'images/weather_icons/' + iconName + '/cloudy.png';
				}
				break;
			case 'snow':
			case 'blowing snow':
				if (windSpeed >= 10) { // wind speed over 10 m/s
					document.getElementById('weathericon').src = 'images/weather_icons/' + iconName + '/snow-windy.png';
				} else {
					document.getElementById('weathericon').src = 'images/weather_icons/' + iconName + '/snow.png';
				}
				break;
			case 'rime':
				if (iconName == 'Tick') {
					document.getElementById('weathericon').src = 'images/weather_icons/' + iconName + '/rime-' + sun_or_moon + '.png';
				} else {
					document.getElementById('weathericon').src = 'images/weather_icons/' + iconName + '/rime.png';
				}
				break;
			default:
				document.getElementById('weathericon').src = 'images/weather_icons/' + iconName + '/' + weather + '.png';
		}
		if ((sun_or_moon !== 'moon') || (weather !== 'sunny')) {
			document.getElementById('stars').style.display = 'none';
		}
	} else {
		document.getElementById('weathericon').src = 'images/weather_icons/' + iconName + '/na.png';
	}
}

function getWeatherData() {
	if (weatherDataRequest) weatherDataRequest.abort();
	
	var currentMonth = today.getMonth() + 1;
	var currentDate = today.getDate();
	var currentHours = today.getHours();
	var currentMinutes = today.getMinutes();
	if (currentMonth < 10)
		currentMonth = '0' + currentMonth;
	if (currentDate < 10)
		currentDate = '0' + currentDate;
	if (currentHours < 10)
		currentHours = '0' + currentHours;
	if (currentMinutes < 10)
		currentMinutes = '0' + currentMinutes;
	var stn_id;
	if (currentPoint == myPoint) {
		stn_id = stationAndLocationCode.AWSstation;
	} else {
		stn_id = stationAndLocationCode2nd.AWSstation;
	}
	var currentWeatherDataURL = weatherDataURL + today.getFullYear() + currentMonth + currentDate + currentHours + currentMinutes + '&0&MINDB_01M&' + stn_id + '&a';
	
	weatherDataRequest = new XMLHttpRequest();
	weatherDataRequest.onreadystatechange = updateWeatherData;
	weatherDataRequest.open('GET', currentWeatherDataURL, true);
	weatherDataRequest.send(null);
}

function updateWeatherData() {
	try {
		if (weatherDataRequest.readyState == 4 && weatherDataRequest.status == 200) { // OK everything
			var response = weatherDataRequest.responseText;
      weatherDataRequest = null;
			var currentData = [];
			currentData = response.match(updateWeatherDataRegEx);
			var tempStorage = currentData[0].replace(updateWeatherDataRegEx2, ','); // do some cleaning
			weatherData = tempStorage.split(',');

			showWeatherIcon();
			var temperature = parseFloat(weatherData[7]);
			if (isNaN(temperature)) {
			  temperature = 'N/A';
		  }
			var windSpeed = parseFloat(weatherData[10]);
			
			if (weatherData[1].match(/blue/)) { // the rainfall detect sensor indicates it's raining
				startRainAlertAnimation();
				showAndStopSnowFalling();
			} else {
				endRainAlertAnimation();
				if (actualSnowFallingEffect && isSnowing) {
					snowStorm.toggleSnow();
					isSnowing = false;
				}
			}

      var decimalPoint;
			if (temperature % 1 === 0) {
				decimalPoint = '.0';
			} else {
				decimalPoint = '';
			}
			
			var tempDiv = document.getElementById('temperature');
			if (!isNaN(temperature)) {
        tempDiv.innerText = convertToFahrenheit(temperature) + decimalPoint + 'º';
        tempDiv.style.color = temperatureColor(temperature);
			} else {
			  tempDiv.innerText = temperature;
			}
			
			if (!isTiger && !isNaN(temperature)) {
				var feelsLike = getFeelsLike(temperature, windSpeed);
				if (feelsLike !== temperature) {
					tempDiv = document.getElementById('feels-like');
					tempDiv.innerText = convertToFahrenheit(feelsLike) + 'º';
					tempDiv.style.color = temperatureColor(feelsLike);
					document.getElementById('paper-plane').style.display = 'block';
				} else {
					document.getElementById('feels-like').innerText = '';
					document.getElementById('paper-plane').style.display = 'none';
				}
			}
				
			if ((weatherData[9] != '&nbsp;') || (weatherData[10] != '&nbsp;')) { // wind data are not empty
				// how strong wind speed is?
				var windColor; // wind speed text color
				
				if ((0 < windSpeed) && (windSpeed < 5)) {
					windForce = 'weak';
					windColor = 'yellow';
				} else if ((5 <= windSpeed) && (windSpeed < 10)) {
					windForce = 'medium';
					windColor = 'cyan';
				} else if (10 <= windSpeed) {
					windForce = 'strong';
					windColor = 'red';
				} else {
					windForce = 'calm';
					windColor = 'white';
				}
				
				// wind directions
				switch (weatherData[9]) {
					case 'N':   // north
						windDirection = 0;
						break;
					case 'NNE':
						windDirection = 22.5;
						break;
					case 'NE':
						windDirection = 45;
						break;
					case 'ENE':
						windDirection = 67.5;
						break;
					case 'E':   // east
						windDirection = 90;
						break;
					case 'ESE':
						windDirection = 112.5;
						break;
					case 'SE':
						windDirection = 135;
						break;
					case 'SSE':
						windDirection = 157.5;
						break;
					case 'S':		// south
						windDirection = 180;
						break;
					case 'SSW':
						windDirection = 202.5;
						break;
					case 'SW':
						windDirection = 225;
						break;
					case 'WSW':
						windDirection = 247.5;
						break;
					case 'W':		// west
						windDirection = 270;
						break;
					case 'WNW':
						windDirection = 292.5;
						break;
					case 'NW':
						windDirection = 315;
						break;
					case 'NNW':
						windDirection = 337.5;
						break;
					default:
						windDirection = 0;
				}
				
				originalAngle = windDirection + sweepbackMax;
				var angle = originalAngle / 180 * Math.PI; // all angles computed in radians
				drawWindVane(angle);
				makeWindVaneSproing();
								
				document.getElementById('wind-speed').innerText =  windSpeed + 'm/s';
				document.getElementById('wind-speed').style.color =  windColor;
			} else {
				document.getElementById('wind-dir').style.display = 'none';
				document.getElementById('wind-speed').style.display = 'none';
			}

			if ((weatherData[14] != '.') && (Boolean(weatherData[14]) === true)) { // humidity info is not empty
				document.getElementById('humidity').innerText = weatherData[14] + '%';
			} else if (humidity !== undefined) { // use humidity info from the current weather xml data instead
				document.getElementById('humidity').innerText = humidity + '%';
			} else {
				document.getElementById('humidity').innerText = '- %';
			}
		}
	}
	catch(exception) {
		drawCheckFailure(getLocalizedString('Failed') + exception);
	}
}

function getFeelsLike(temp, wind) {
	var T = parseFloat(temp);
	var W = parseFloat(wind);
	var result = 0.0;
	if ( T <= 10 && W > 4.8 ) {
			W = Math.pow(W,0.16);
			result = Math.round(13.12 + 0.6215 * T - 11.37 * W + 0.3965 * W * T);
			if (result > T) {
				result = T;
			}
	} else {
		result = T;
	}
	return result;
}

function temperatureColor(temp) {
	if (temp > 0) { // if the temperature is above zero
		if (temp >= 30) {
			return '#FF8000';
		} else if (temp >= 20) {
			return 'yellow';
		} else {
			return 'white';
		}
	} else {
	  return 'cyan';
	}
}

// Retrieve the sunrise and sunset time of today
function getSunTime() {
  var longitude, latitude;
	if (currentPoint == myPoint) {
		longitude = stationAndLocationCode.longitude.split('/');
		latitude = stationAndLocationCode.latitude.split('/');
	} else {
		longitude = stationAndLocationCode2nd.longitude.split('/');
		latitude = stationAndLocationCode2nd.latitude.split('/');
	}
	var year = today.getFullYear();
	var month = today.getMonth();
	var day = today.getDate();
	var currentSunData = SunriseSunset(month, day, year, -9, +longitude[0], +longitude[1], +longitude[2], +latitude[0], +latitude[1], +latitude[2]);
	currentSunData.timeTag = today.toDateString()+currentPoint.z3;
	document.getElementById('risenfall').innerHTML = '&uarr;' + currentSunData.riseHours + ':' + currentSunData.riseMinutes + ' &darr;' + currentSunData.setHours + ':' + currentSunData.setMinutes;
	
	if (currentPoint == myPoint) {
	  sunDataPt1 = currentSunData;
	} else {
	  sunDataPt2 = currentSunData;
  }
}

// Found out if it's daytime or night
function getDayNight() {
	var currentHours = today.getHours();
	var currentMinutes = today.getMinutes();
	var sunData = (currentPoint == myPoint) ? sunDataPt1 : sunDataPt2;
	
	// there must be a better way doing this, hmmm (->today.getTime())
	if (((currentHours > sunData.riseHours) && (currentHours < sunData.setHours)) || 
		((currentHours == sunData.riseHours) && (currentMinutes >= sunData.riseMinutes)) || 
			((currentHours == sunData.setHours) && (currentMinutes <= sunData.setMinutes))) {
		return 'sun';
	} else {
		return 'moon';
	}
}

// Return the sky value for the background images
function getSkyInfo() {
	var sunData = (currentPoint == myPoint) ? sunDataPt1 : sunDataPt2;
		
	var now = today.getHours() * 60 + today.getMinutes();
	var sunrise = sunData.riseHours * 60 + parseInt(sunData.riseMinutes, 10);
	var sunset = sunData.setHours * 60 + parseInt(sunData.setMinutes, 10);
	
	if ((now - sunrise <= 30 ) && (now - sunrise >= -30 )) { // If it's around sunrise 
		return 'near-sunrise';
	} else if ((now - sunset >= -30 ) && (now - sunset <= 30 )) { // or sunset time
		return 'near-sunset';
	} else if (sun_or_moon == 'moon') {
		return 'nighttime';
	} else {
		return 'daytime';
	}
}

function updateMoonPhases() {
	var lunarDateToday = lunarDate[today.getFullYear()][(today.getMonth() + 1)][(today.getDate()-1)];
	document.getElementById('moon-phase-icon').src = 'images/weather_icons/MoonPhases/' + lunarDateToday.split('-')[2] + '.png';
	document.getElementById('lunar-date').innerText = lunarDateToday;
}

function changeWindIcon() {
	actualWindIconIndex = (actualWindIconIndex === 0) ? 1 : 0;
	widget.setPreferenceForKey(actualWindIconIndex, createkey('windiconindex'));
	document.getElementById('wind-dir').style.backgroundImage = 'url(images/wind_icons/' + windIcon[actualWindIconIndex].name + '/wind_bg.png)';
	originalAngle = windDirection + sweepbackMax;
	var angle = originalAngle / 180 * Math.PI; // all angles computed in radians
	drawWindVane(angle);
	makeSproing();
}

function drawWindVane(angle) {
	var canvas = document.getElementById('wind-dir');
	var context = canvas.getContext('2d');
	var w, h;
	if (windForce !== 'calm') {
		w = windIcon[actualWindIconIndex].width;
		h = windIcon[actualWindIconIndex].height;
	} else {
		w = 25;
		h = 25;
	}
	var windVane = new Image(w, h);
	windVane.src = 'images/wind_icons/' + windIcon[actualWindIconIndex].name + '/' + windForce + '_wind_vane.png';
	windVane.onload = function() {
    context.clearRect(0, 0, 48, 48);
	
    context.save();
    context.translate(48/2, 48/2); // to rotate the image around the center
	
    context.rotate(angle);
    context.drawImage(windVane, -w/2, -h/2);

    context.restore();
	};
}

// wind vane "sproing" animation
function makeWindVaneSproing() {
	if (windForce !== 'calm') {
		if ( sweepbackAnimation.timer !== null ) {
			clearInterval( sweepbackAnimation.timer );
			sweepbackAnimation.timer = null;
		}
		sweepbackAnimation.step = 0;
		sweepbackAnimation.timer = setInterval(sweepbackAnimate, sweepbackAnimateInterval);
	}
}

function sweepbackAnimate() {
	if ( sweepbackAnimation.step >= sweepbackAmounts.length ) {
		//  no more steps to the animation, so clear the timer
		clearInterval(sweepbackAnimation.timer);
		sweepbackAnimation.timer = null;
		sweepbackAnimation.step = 0;
	} else {
		originalAngle = originalAngle + sweepbackAmounts[sweepbackAnimation.step];
		var angle = originalAngle / 180 * Math.PI; // all angles computed in radians
		
		drawWindVane(angle);
		
		sweepbackAnimation.step++;
	}
}

function getShortTermForecast() {
	if (shortTermForecastRequest) shortTermForecastRequest.abort();
	var currentAreaForecastURL = shortTermForecastURL + 'gridx=' + townMappingData[currentPoint.z3].x + '&gridy=' + townMappingData[currentPoint.z3].y;
	
	shortTermForecastRequest = new XMLHttpRequest();
	shortTermForecastRequest.onreadystatechange = updateShortTermForecast;
	if (!isTiger) {
		weatherRequest.overrideMimeType('text/xml');
	}
	shortTermForecastRequest.open('GET', currentAreaForecastURL, true);
	shortTermForecastRequest.send(null);
}

function updateShortTermForecast() {
	try {
		if (shortTermForecastRequest.readyState == 4 && shortTermForecastRequest.status == 200) { // OK everything
		  var response, result, i, j, len, timeText, rainElement, temperatureElement;
			if (!isTiger) {
				response = shortTermForecastRequest.responseXML;
				result = response.evaluate('/wid/body/data/hour', response, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
				if (result.snapshotLength !== 0) {
					timeElement = [];
					for (i=0, len=result.snapshotLength; i < len; i++) {
					  timeText = result.snapshotItem(i).textContent;
						timeElement[i] = (timeText.length == 1) ? '0' + timeText : timeText;
				  }
				}
				result = response.evaluate('/wid/body/data/wfEn', response, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
				if (result.snapshotLength !== 0) {
					weatherElement = [];
					for (i=0, len=result.snapshotLength; i < len; i++) {
						weatherElement[i] = result.snapshotItem(i).textContent.toLowerCase();
					}
				}
				result = response.evaluate('/wid/body/data/pop', response, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
				if (result.snapshotLength !== 0) {
					rainElement = [];
					for (i=0, len=result.snapshotLength; i < len; i++) {
						rainElement[i] = result.snapshotItem(i).textContent;
					}
				}
				result = response.evaluate('/wid/body/data/temp', response, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
				if (result.snapshotLength !== 0) {
					temperatureElement = [];
					for (i=0, len=result.snapshotLength; i < len; i++) {
						temperatureElement[i] = Math.round(result.snapshotItem(i).textContent);
					}
				}
			} else {
				response = shortTermForecastRequest.responseText;
				result = getDOMfromXML(response);
				var nodes = result.getElementsByTagName('hour');
				if (nodes.length !== 0) {
					timeElement = [];
					for (i=0, len=node.length; i < len; i++) {
					  timeText = nodes[i].firstChild.data;
						timeElement[i] = (timeText.length == 1) ? '0' + timeText : timeText;
				  }
				}
				nodes = result.getElementsByTagName('wfEn');
				if (nodes.length !== 0) {
					weatherElement = [];
					for (i=0, len=node.length; i < len; i++) {
						weatherElement[i] = nodes[i].firstChild.data.toLowerCase();
					}
				}
				nodes = result.getElementsByTagName('pop');
				if (nodes.length !== 0) {
					rainElement = [];
					for (i=0, len=node.length; i < len; i++) {
						rainElement[i] = nodes[i].firstChild.data;
					}
				}
				nodes = result.getElementsByTagName('temp');
				if (nodes.length !== 0) {
					temperatureElement = [];
					for (i=0, len=node.length; i < len; i++) {
						temperatureElement[i] = Math.round(nodes[i].firstChild.data);
					}
				}
			}
			shortTermForecastRequest = null;
			
			// Calculate the number of today's elements
			if (timeElement[0] == '03') {
				todayLength = 0;
			} else {
				for (i=0, len=timeElement.length; i < len; i++) {
					if (timeElement[i] == '24') {
						todayLength = i+1;
						break;
					}
				}
			}
			
			var doc = document;
			var temporaryTimeDiv = [], temporaryRainDiv = [], temporaryTempDiv = [], timeClass;
			if (todayLength !== 0) {
				for (i = 0; i < todayLength; i++) {
				  timeClass = 'time ' + getTimeClass(timeElement[i]);
					temporaryTimeDiv.push('<span class="' + timeClass + '">'  + timeElement[i] + '<\/span>');
					temporaryRainDiv.push('<span class="text" style="color:' + rainColor(rainElement[i]) + '">' + rainElement[i] + '<\/span>');
					temporaryTempDiv.push('<span class="text" style="color:' + temperatureColor(temperatureElement[i]) + '">' + convertToFahrenheit(temperatureElement[i]) + '<\/span>');
				}
				doc.getElementById('time-today').innerHTML = temporaryTimeDiv.join('');
				doc.getElementById('day1').style.width = todayLength * 20 + 'px';
				doc.getElementById('rain-probability-today').innerHTML = temporaryRainDiv.join('');
				doc.getElementById('today-hi-low').innerHTML = temporaryTempDiv.join('');
			} else {
				doc.getElementById('day1').style.width = 0;
			}
			
			temporaryTimeDiv = []; temporaryRainDiv = []; temporaryTempDiv = [];
			for (i = todayLength, j = todayLength + 8; i < j; i++) {
				timeClass = 'time ' + getTimeClass(timeElement[i]);
				temporaryTimeDiv.push('<span class="' + timeClass + '">'  + timeElement[i] + '<\/span>');
				temporaryRainDiv.push('<span class="text" style="color:' + rainColor(rainElement[i]) + '">' + rainElement[i] + '<\/span>');
				temporaryTempDiv.push('<span class="text" style="color:' + temperatureColor(temperatureElement[i]) + '">' + convertToFahrenheit(temperatureElement[i]) + '<\/span>');
			} 
			doc.getElementById('time-tomorrow').innerHTML = temporaryTimeDiv.join('');
			doc.getElementById('rain-probability-tomorrow').innerHTML = temporaryRainDiv.join('');
			doc.getElementById('tomorrow-hi-low').innerHTML = temporaryTempDiv.join('');
			
			temporaryTimeDiv = []; temporaryRainDiv = []; temporaryTempDiv = [];
			for (i = todayLength + 8, j = timeElement.length; i < j; i++) {
				timeClass = 'time ' + getTimeClass(timeElement[i]);
				temporaryTimeDiv.push('<span class="' + timeClass + '">'  + timeElement[i] + '<\/span>');
				temporaryRainDiv.push('<span class="text" style="color:' + rainColor(rainElement[i]) + '">' + rainElement[i] + '<\/span>');
				temporaryTempDiv.push('<span class="text" style="color:' + temperatureColor(temperatureElement[i]) + '">' + convertToFahrenheit(temperatureElement[i]) + '<\/span>');
			}
			doc.getElementById('day3').style.width = ((timeElement.length - (todayLength + 8)) * 20) + "px";
			doc.getElementById('time-after-tomorrow').innerHTML = temporaryTimeDiv.join('');
			doc.getElementById('rain-probability-after-tomorrow').innerHTML = temporaryRainDiv.join('');
			doc.getElementById('after-tomorrow-hi-low').innerHTML = temporaryTempDiv.join('');
			
			// fill in the date columns
			var dayOfToday = today.getDay();
			var dateTodayDiv = doc.getElementById('date-today');
			var dateTomorrowDiv = doc.getElementById('date-tomorrow');
			var dateAfterTomorrowDiv = doc.getElementById('date-after-tomorrow');

			if ((todayLength === 0) && ((0 <= today.getHours()) && (today.getHours() < 3))) { // it's past midnight, tomorrow is today actually
				dateTomorrowDiv.innerText = 
				getLocalizedString('Today') + '(' + getLocalizedDayOfWeekName(dayOfToday) + ')';
				dateAfterTomorrowDiv.innerText = 
				getLocalizedString('Tomorrow') + '(' + getLocalizedDayOfWeekName(dayOfToday+1) + ')';

				// coloring the text of days
				dateTomorrowDiv.style.color = getDayColor(dayOfToday);
				dateAfterTomorrowDiv.style.color = getDayColor(dayOfToday+1);
			} else {
				if ((todayLength === 0) && (today.getHours() == 23)) { // just before the midnight, don't display the date info if there's no weather data
					dateTodayDiv.innerText = '';
				} else if (todayLength !== 0) {
					dateTodayDiv.innerText = 
					getLocalizedString('Today') + ' (' + getLocalizedDayOfWeekName(dayOfToday) + ')';
					dateTodayDiv.style.color = getDayColor(dayOfToday);
				}

				dateTomorrowDiv.innerText = 
				getLocalizedString('Tomorrow') + ' (' + getLocalizedDayOfWeekName(dayOfToday+1) + ')';
				var dayAfterTomorrow = new Date();
				dayAfterTomorrow.setDate(dayAfterTomorrow.getDate() + 2);
				dateAfterTomorrowDiv.innerText = 
				dayAfterTomorrow.getDate() + ' (' + getLocalizedDayOfWeekName(dayOfToday+2) + ')';

				dateTomorrowDiv.style.color = getDayColor(dayOfToday+1);
				dateAfterTomorrowDiv.style.color = getDayColor(dayOfToday+2);
			}

			showForecastWeatherIcons();
		}
	}
	catch(exception) {
		drawCheckFailure(getLocalizedString('Failed') + exception);
  }
}

function getTimeClass(timeString) {
  switch(timeString) {
    case '06': case '09': case '12': case '15': case '18':
      return 'day';
    default:
      return 'night';
  }
}

function getMidTermForecast() {
  if (midTermForecastRequest) midTermForecastRequest.abort();
	
	midTermForecastRequest = new XMLHttpRequest();
	midTermForecastRequest.onreadystatechange = updateMidTermForecast;
	if (!isTiger) {
		weatherRequest.overrideMimeType('text/xml');
	}
	midTermForecastRequest.open('GET', midTermForecastURL, true);
	midTermForecastRequest.send(null);
}

function updateMidTermForecast() {
  try {
		if (midTermForecastRequest.readyState == 4 && midTermForecastRequest.status == 200) { // OK everything
		  midTermForecastLastUpdated = today;
		  var response, result, cityCode, i, j, len, dateElement, weatherConditionElement, temperatureLowElement, temperatureHiElement;

		  if (currentPoint == myPoint) {
        cityCode = stationAndLocationCode.midTermForecastCityCode;
      } else {
        cityCode = stationAndLocationCode2nd.midTermForecastCityCode;
      }

			response = midTermForecastRequest.responseXML;
			result = response.evaluate('/wid/body/location[@city="' + cityCode + '"]/data/tmEf', response, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
			if (result.snapshotLength !== 0) {
				dateElement = [];
				for (i=0, len=result.snapshotLength; i < len; i++) {
					dateElement[i] = result.snapshotItem(i).textContent.replace(/\d{4}-\d{2}-/, '');
			  }
			}
			result = response.evaluate('/wid/body/location[@city="' + cityCode + '"]/data/wf', response, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
			if (result.snapshotLength !== 0) {
				weatherConditionElement = [];
				for (i=0, len=result.snapshotLength; i < len; i++) {
					weatherConditionElement[i] = translateKoToEn(result.snapshotItem(i).textContent);
			  }
			}
			result = response.evaluate('/wid/body/location[@city="' + cityCode + '"]/data/tmn', response, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
			if (result.snapshotLength !== 0) {
				temperatureLowElement = [];
				for (i=0, len=result.snapshotLength; i < len; i++) {
					temperatureLowElement[i] = result.snapshotItem(i).textContent;
				}
			}
			result = response.evaluate('/wid/body/location[@city="' + cityCode + '"]/data/tmx', response, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
			if (result.snapshotLength !== 0) {
				temperatureHiElement = [];
				for (i=0, len=result.snapshotLength; i < len; i++) {
					temperatureHiElement[i] = result.snapshotItem(i).textContent;
				}
			}
			midTermForecastRequest = null;
			
			var doc = document;
			var dayOfToday = today.getDay();
			if (((todayLength === 0) && (today.getHours() === 23)) || ((0 <= today.getHours()) && (today.getHours() < 6))) {
        dayOfToday = dayOfToday - 1;
      }
      var dateID, weatherID, theWeatherIcon, hiLowID;
      for (i = 0; i < 6; i++) {
        dateID = 'date' + (i + 1);
        var theDate = doc.getElementById(dateID);
        theDate.innerText = dateElement[i];
        theDate.style.color = getDayColor(dayOfToday+2+i);

        weatherID = 'weather' + (i + 1);
        theWeatherIcon = getForecastWeatherIcon(weatherConditionElement[i], 12);
        doc.getElementById(weatherID).innerHTML = '<img src="images/weather_icons/' + iconName + '/small_icons/' + theWeatherIcon + '.png" width="20" height="20" \/>';
        if (weatherConditionElement[i].match(/rain|snow/)) {
          theDate.style.borderBottomColor = '#ff0a00';
        } else {
          theDate.style.borderBottomColor = '#2f099d';
        }

        hiLowID = 'hi-low' + (i + 1);
        doc.getElementById(hiLowID).innerHTML = '<span style="color:aqua;float:left;">' + convertToFahrenheit(temperatureLowElement[i]) + '<\/span><br><span style="color:red;float:right;">' + convertToFahrenheit(temperatureHiElement[i]) + '<\/span>';
			}
		}
	}
	catch(exception) {
		drawCheckFailure(getLocalizedString('Failed') + exception);
  }
}

function rainColor(rainProb) {
	if (rainProb > 50) {
		return '#FF0080';
	} else if (rainProb > 40) {
		return '#FF00FF';
	} else {
		return '#66CCFF';
	}
}

function showForecastWeatherIcons() {
	var temporaryDiv = '', divToPushIn = '', theWeatherIcon = '', i, j;
	var doc = document;
	for (i = 0; i < todayLength; i++) {
		theWeatherIcon = getForecastWeatherIcon(weatherElement[i], timeElement[i]);
		divToPushIn = '<img src="images/weather_icons/' + iconName + '/small_icons/' + theWeatherIcon + '.png" title="' + localizedWeatherName[theWeatherIcon] + '" width="16" height="16" \/>';
		temporaryDiv = temporaryDiv + divToPushIn;
	}
	doc.getElementById('weather-today').innerHTML = temporaryDiv;

	temporaryDiv = '';
	divToPushIn = '';
	for (i = todayLength; i < todayLength + 8; i++) {
		theWeatherIcon = getForecastWeatherIcon(weatherElement[i], timeElement[i]);
		divToPushIn = '<img src="images/weather_icons/' + iconName + '/small_icons/' + theWeatherIcon + '.png" title="' + localizedWeatherName[theWeatherIcon] + '" width="16" height="16" \/>';
		temporaryDiv = temporaryDiv + divToPushIn;
	}
	doc.getElementById('weather-tomorrow').innerHTML = temporaryDiv;

	temporaryDiv = '';
	divToPushIn = '';
	for (i = todayLength + 8, j = weatherElement.length; i < j; i++) {
		theWeatherIcon = getForecastWeatherIcon(weatherElement[i], timeElement[i]);
		divToPushIn = '<img src="images/weather_icons/' + iconName + '/small_icons/' + theWeatherIcon + '.png" title="' + localizedWeatherName[theWeatherIcon] + '" width="16" height="16" \/>';
		temporaryDiv = temporaryDiv + divToPushIn;
	}
	doc.getElementById('weather-after-tomorrow').innerHTML = temporaryDiv;
}

function updateMidTermForecastIcon() {
  var weatherID, weatherDiv;
  for (i = 0; i < 6; i++) {
    weatherID = 'weather' + (i + 1);
    weatherDiv = document.getElementById(weatherID);
    var divToReplace = weatherDiv.innerHTML;
    divToReplace = divToReplace.replace(/weather_icons\/.*\/small_icons/, 'weather_icons/' + iconName + '/small_icons');
    weatherDiv.innerHTML = divToReplace;
  }
}

function getDayColor(day) {
	switch(day%7) {
		case 6: return '#00FFFF'; // Saturday
		case 0: return 'red'; // Sunday
		default: return 'white';
	}
}

function getStationAndLocationCode(Point) {
	var stationAndLocationCode;
	var currentPoint = Point.z2;
	
	switch (currentPoint) {
		// Seoul
    case '11110': case '11140': case '11170': case '11200': case '11215':
    case '11230': case '11260': case '11290': case '11305': case '11320':
		case '11350': case '11380': case '11410': case '11440': case '11470':
		case '11500': case '11530': case '11545': case '11560': case '11620':
		case '11650': case '11680': case '11710': case '11740': case '11590':
		stationAndLocationCode = {'station': '108', 'longitude': '-126/58/1', 'latitude':'37/32/59', 'midTermForecastCityCode':'11B10101'}; break;
		// 구리시
		case '41310':
		stationAndLocationCode =  {'station': '108', 'longitude': '-127/9/0', 'latitude':'37/35/59', 'midTermForecastCityCode':'11B10101'}; break;
		// 의정부시
		case '41150':
		stationAndLocationCode =  {'station': '108', 'longitude': '-127/3/0', 'latitude':'37/43/58', 'midTermForecastCityCode':'11B10101'}; break;
		// 과천시
		case '41290':
		stationAndLocationCode =  {'station': '108', 'longitude': '-126/58/58', 'latitude':'37/25/58', 'midTermForecastCityCode':'11B10101'}; break;
		// 성남시
		case '41131':
		stationAndLocationCode =  {'station': '108', 'longitude': '-127/7/58', 'latitude':'37/25/1', 'midTermForecastCityCode':'11B10101'}; break;
		// 고양시
		case '41281': case '41285': case '41287':
		stationAndLocationCode =  {'station': '108', 'longitude': '-126/49/58', 'latitude':'37/39/0', 'midTermForecastCityCode':'11B10101'}; break;
		// 남양주시
		case '41360':
		stationAndLocationCode =  {'station': '108', 'longitude': '-127/11/59', 'latitude':'37/37/1', 'midTermForecastCityCode':'11B10101'}; break;
		// 하남시
		case '41450':
		stationAndLocationCode =  {'station': '108', 'longitude': '-127/13/1', 'latitude':'37/31/1', 'midTermForecastCityCode':'11B10101'}; break;
		// 광명시
		case '41210':
		stationAndLocationCode =  {'station': '108', 'longitude': '-126/50/59', 'latitude':'37/27/0', 'midTermForecastCityCode':'11B10101'}; break;
		
		// Busan
		case '26110': case '26140': case '26170': case '26200': case '26230':
    case '26260': case '26290': case '26320': case '26350': case '26380':
		case '26410': case '26440': case '26470': case '26500': case '26530':
		case '26710':
		stationAndLocationCode =  {'station': '159', 'longitude': '-129/1/58', 'latitude':'35/5/59', 'midTermForecastCityCode':'11H20201'}; break;
		// Gimhae
		case '48250':
		stationAndLocationCode =  {'station': '159', 'longitude': '-128/52/58', 'latitude':'35/13/1', 'midTermForecastCityCode':'11H20201'}; break;
		// Yangsan
		case '48330': 
		stationAndLocationCode =  {'station': '159', 'longitude': '-129/1/58', 'latitude':'35/20/59', 'midTermForecastCityCode':'11H20201'}; break;
		
		// Daegu
		case '27110': case '27140': case '27170': case '27200': case '27230':
    case '27260': case '27290': case '27710':
		stationAndLocationCode =  {'station': '143', 'longitude': '-128/34/58', 'latitude':'35/50/59', 'midTermForecastCityCode':'11H10701'}; break;
		
		// Incheon
		case '28110': case '28140': case '28170': case '28185': case '28200':
    case '28237': case '28245': case '28260':
		stationAndLocationCode =  {'station': '112', 'longitude': '-126/37/58', 'latitude':'37/27/0', 'midTermForecastCityCode':'11B20201'}; break;
		// 부천시
		case '41195': case '41197': case '41199':
		stationAndLocationCode =  {'station': '112', 'longitude': '-126/46/58', 'latitude':'37/28/1', 'midTermForecastCityCode':'11B20201'}; break;
		// 시흥시
		case '41390':
		stationAndLocationCode =  {'station': '112', 'longitude': '-126/46/58', 'latitude':'37/25/58', 'midTermForecastCityCode':'11B20201'}; break;
		// 안산시
		case '41271': case '41273':
		stationAndLocationCode =  {'station': '112', 'longitude': '-126/49/1', 'latitude':'37/19/1', 'midTermForecastCityCode':'11B20201'}; break;
		// 김포시
		case '41570':
		stationAndLocationCode =  {'station': '112', 'longitude': '-126/41/59', 'latitude':'37/37/1', 'midTermForecastCityCode':'11B20201'}; break;
		
		// Ganghwa
		case '28710': 
		stationAndLocationCode =  {'station': '201', 'nearCityStation': '112', 'longitude': '-126/28/58', 'latitude':'37/43/58', 'midTermForecastCityCode':'11B20201'}; break;
		
		// Baengnyeongdo
		case '28720': 
		stationAndLocationCode =  {'station': '102', 'longitude': '-124/39/21', 'latitude':'37/57/3', 'midTermForecastCityCode':'11B20201'}; break;
		
		// Gwangju
		case '29110': case '29140': case '29155': case '29170': case '29200':
		stationAndLocationCode =  {'station': '156', 'longitude': '-126/54/0', 'latitude':'35/9/0', 'midTermForecastCityCode':'11F20501'}; break;
		
		// Daejeon
		case '30110': case '30140': case '30170': case '30200': case '30230':
		stationAndLocationCode =  {'station': '133', 'longitude': '-127/25/1', 'latitude':'36/19/1', 'midTermForecastCityCode':'11C20401'}; break;
		
		// Ulsan
		case '31110': case '31140': case '31170': case '31200': case '31710':
		stationAndLocationCode =  {'station': '152', 'longitude': '-129/19/1', 'latitude':'35/31/1', 'midTermForecastCityCode':'11H20101'}; break;
		// Miryang
		case '48270':
		stationAndLocationCode =  {'station': '152', 'longitude': '-128/45/0', 'latitude':'35/28/58', 'midTermForecastCityCode':'11H20101'}; break;
		
		// Suwon
		case '41111': case '41113': case '41115': case '41117': case '41133': 
		case '41135': case '41610':
		stationAndLocationCode =  {'station': '119', 'longitude': '-127/1/58', 'latitude':'37/16/1', 'midTermForecastCityCode':'11B20601'}; break;
		// 군포시
		case '41410':
		stationAndLocationCode =  {'station': '119', 'longitude': '-126/55/58', 'latitude':'37/27/0', 'midTermForecastCityCode':'11B20601'}; break;
		// 안양시
		case '41171': case '41173':
		stationAndLocationCode =  {'station': '119', 'longitude': '-126/55/1', 'latitude':'37/22/58', 'midTermForecastCityCode':'11B20601'}; break;
		// 오산시
		case '41370':
		stationAndLocationCode =  {'station': '119', 'longitude': '-127/4/58', 'latitude':'37/9/0', 'midTermForecastCityCode':'11B20601'}; break;
		// 의왕시
		case '41430':
		stationAndLocationCode =  {'station': '119', 'longitude': '-126/56/59', 'latitude':'37/19/58', 'midTermForecastCityCode':'11B20601'}; break;
		// 평택시
		case '41220':
		stationAndLocationCode =  {'station': '119', 'longitude': '-127/5/59', 'latitude':'36/59/0', 'midTermForecastCityCode':'11B20601'}; break;
		// 화성시
		case '41590':
		stationAndLocationCode =  {'station': '119', 'longitude': '-127/1/1', 'latitude':'37/13/1', 'midTermForecastCityCode':'11B20601'}; break;
		// 용인시
		case '41461': case '41463': case '41465':
		stationAndLocationCode =  {'station': '119', 'longitude': '-127/13/1', 'latitude':'38/4/1', 'midTermForecastCityCode':'11B20601'}; break;
		// 안성시
		case '41550':
		stationAndLocationCode =  {'station': '119', 'longitude': '-127/16/58', 'latitude':'37/0/0', 'midTermForecastCityCode':'11B20601'}; break;
		
		// Dongducheon
		case '41250':
		stationAndLocationCode =  {'station': '98', 'nearCityStation': '108', 'longitude': '-127/4/1', 'latitude':'37/54/0', 'midTermForecastCityCode':'11B20305'}; break;
		// 양주시
		case '41630':
		stationAndLocationCode =  {'station': '98', 'nearCityStation': '108', 'longitude': '-127/4/1', 'latitude':'37/49/58', 'midTermForecastCityCode':'11B10101'}; break;
		// 연천군
		case '41800':
		stationAndLocationCode =  {'station': '98', 'nearCityStation': '108', 'longitude': '-127/4/58', 'latitude':'38/4/1', 'midTermForecastCityCode':'11B10101'}; break;
		// 포천시
		case '41650':
		stationAndLocationCode =  {'station': '98', 'nearCityStation': '108', 'longitude': '-127/11/59', 'latitude':'37/52/58', 'midTermForecastCityCode':'11B20305'}; break;
		
		// Icheon
		case '41500': case '41550':
		stationAndLocationCode =  {'station': '203', 'nearCityStation': '119', 'longitude': '-127/26/59', 'latitude':'37/16/1', 'midTermForecastCityCode':'11B20601'}; break;
		// 여주군
		case '41730':
		stationAndLocationCode =  {'station': '203', 'nearCityStation': '114', 'longitude': '-127/37/58', 'latitude':'37/16/58', 'midTermForecastCityCode':'11B20601'}; break;
        
		// 파주시
		case '41480':
		stationAndLocationCode =  {'station': '99', 'nearCityStation': '108', 'longitude': '-126/46/58', 'latitude':'37/43/58', 'midTermForecastCityCode':'11B20305'}; break;
		
		// Yangpyeong
		case '41830':
		stationAndLocationCode =  {'station': '202', 'nearCityStation': '108', 'longitude': '-127/30/0', 'latitude':'37/28/1', 'midTermForecastCityCode':'11B20601'}; break;
		
		// Chuncheon
		case '42110':
		stationAndLocationCode =  {'station': '101', 'longitude': '-127/43/58', 'latitude':'37/52/58', 'midTermForecastCityCode':'11D10301'}; break;
		// 가평군
		case '41820':
		stationAndLocationCode =  {'station': '101', 'longitude': '-127/31/58', 'latitude':'37/49/58', 'midTermForecastCityCode':'11D10301'}; break;
		// Hongcheon
		case '42720': 
		stationAndLocationCode =  {'station': '101', 'longitude': '-127/52/58', 'latitude':'37/40/58', 'midTermForecastCityCode':'11D10301'}; break;
		
		// Wonju
		case '42130': 
		stationAndLocationCode =  {'station': '114', 'longitude': '-127/56/59', 'latitude':'37/19/58', 'midTermForecastCityCode':'11D10301'}; break;
		// 횡성군
		case '42730':
		stationAndLocationCode =  {'station': '114', 'longitude': '-127/58/58', 'latitude':'37/28/58', 'midTermForecastCityCode':'11D10301'}; break;
		
		// Gangneung
		case '42150': 
		stationAndLocationCode =  {'station': '105', 'longitude': '-128/52/58', 'latitude':'37/45/0', 'midTermForecastCityCode':'11D20501'}; break;
		
		// Donghae
		case '42170': 
		stationAndLocationCode =  {'station': '106', 'nearCityStation': '105', 'longitude': '-129/7/1', 'latitude':'37/18/36', 'midTermForecastCityCode':'11D20501'}; break;
		
		// Taebaek
		case '42190': 
		stationAndLocationCode =  {'station': '216', 'nearCityStation': '105', 'longitude': '-128/58/58', 'latitude':'37/13/58', 'midTermForecastCityCode':'11D20501'}; break;
		// 삼척시
		case '42230':
		stationAndLocationCode =  {'station': '216', 'nearCityStation': '105', 'longitude': '-129/10/1', 'latitude':'37/27/0', 'midTermForecastCityCode':'11D20501'}; break;
		
		// Sokcho
		case '42210':
		stationAndLocationCode =  {'station': '90', 'nearCityStation': '105', 'longitude': '-128/34/58', 'latitude':'38/15/0', 'midTermForecastCityCode':'11D20501'}; break;
		// 강원도 고성군
		case '42820':
		stationAndLocationCode =  {'station': '90', 'nearCityStation': '105', 'longitude': '-128/28/1', 'latitude':'38/13/12', 'midTermForecastCityCode':'11D20501'}; break;
		// 양양군
		case '42830':
		stationAndLocationCode =  {'station': '90', 'nearCityStation': '105', 'longitude': '-128/40/58', 'latitude':'38/4/1', 'midTermForecastCityCode':'11D20501'}; break;
		
		// Yeongwol
		case '42750': 
		stationAndLocationCode =  {'station': '121', 'nearCityStation': '105', 'longitude': '-128/28/1', 'latitude':'37/10/58', 'midTermForecastCityCode':'11D20501'}; break;
		// 정선군
		case '42770':
		stationAndLocationCode =  {'station': '121', 'nearCityStation': '105', 'longitude': '-128/36/25', 'latitude':'37/22/1', 'midTermForecastCityCode':'11D20501'}; break;
		// 평창군
		case '42760':
		stationAndLocationCode =  {'station': '121', 'nearCityStation': '105', 'longitude': '-128/22/58', 'latitude':'37/22/19', 'midTermForecastCityCode':'11D20501'}; break;
		
		// Cheorwon
		case '42780':
		stationAndLocationCode =  {'station': '95', 'nearCityStation': '101', 'longitude': '-127/11/59', 'latitude':'38/13/1', 'midTermForecastCityCode':'11D10301'}; break;
		// 화천군
		case '42790': 
		stationAndLocationCode =  {'station': '95', 'nearCityStation': '101', 'longitude': '-127/41/59', 'latitude':'38/5/59', 'midTermForecastCityCode':'11D10301'}; break;
		
		// 양구군
		case '42800': 
		stationAndLocationCode =  {'station': '211', 'nearCityStation': '101', 'longitude': '-127/58/58', 'latitude':'38/5/59', 'midTermForecastCityCode':'11D10301'}; break;
		// 인제군
		case '42810':
		stationAndLocationCode =  {'station': '211', 'nearCityStation': '101', 'longitude': '-128/10/1', 'latitude':'38/4/1', 'midTermForecastCityCode':'11D10301'}; break;
		
		// Cheongju
		case '43111': case '43113': case '43745': case '43745': case '43710':
		stationAndLocationCode =  {'station': '131', 'longitude': '-127/28/58', 'latitude':'36/37/58', 'midTermForecastCityCode':'11C10301'}; break;
		// 괴산군
		case '43760':
		stationAndLocationCode =  {'station': '131', 'longitude': '-127/45/0', 'latitude':'36/47/59', 'midTermForecastCityCode':'11C10301'}; break;
		// Boeun
		case '43720': 
		stationAndLocationCode =  {'station': '131', 'longitude': '-127/43/1', 'latitude':'36/25/58', 'midTermForecastCityCode':'11C10301'}; break;
		
		// Chungju
		case '43130': 
		stationAndLocationCode =  {'station': '127', 'nearCityStation': '114', 'longitude': '-127/55/1', 'latitude':'36/58/1', 'midTermForecastCityCode':'11C10301'}; break;
		// 음성군
		case '43770':
		stationAndLocationCode =  {'station': '127', 'nearCityStation': '131', 'longitude': '-127/40/58', 'latitude':'36/55/58', 'midTermForecastCityCode':'11C10301'}; break;
		// 진천군
		case '43750':
		stationAndLocationCode =  {'station': '127', 'nearCityStation': '131', 'longitude': '-127/25/58', 'latitude':'36/50/59', 'midTermForecastCityCode':'11C10301'}; break;
		// Jecheon
		case '43150':
		stationAndLocationCode =  {'station': '127', 'nearCityStation': '114', 'longitude': '-128/11/59', 'latitude':'37/7/1', 'midTermForecastCityCode':'11C10301'}; break;
		// 단양군
		case '43800':
		stationAndLocationCode =  {'station': '127', 'nearCityStation': '114', 'longitude': '-128/21/0', 'latitude':'36/28/58', 'midTermForecastCityCode':'11C10301'}; break;
		
		// 영동군 (추풍령)
		case '43740':
		stationAndLocationCode =  {'station': '135', 'nearCityStation': '133', 'longitude': '-127/46/1', 'latitude':'36/10/1', 'midTermForecastCityCode':'11C20401'}; break;
		// 옥천군
		case '43730':
		stationAndLocationCode =  {'station': '135', 'nearCityStation': '133', 'longitude': '-127/34/1', 'latitude':'36/16/58', 'midTermForecastCityCode':'11C20401'}; break;
		
		// Cheonan
		case '44131': case '44133':
		stationAndLocationCode =  {'station': '232', 'nearCityStation': '129', 'longitude': '-127/9/0', 'latitude':'36/47/59', 'midTermForecastCityCode':'11C20101'}; break;
		// 아산시
		case '44200':
		stationAndLocationCode =  {'station': '232', 'nearCityStation': '129', 'longitude': '-127/1/1', 'latitude':'36/46/58', 'midTermForecastCityCode':'11C20101'}; break;
		// 예산군
		case '44810':
		stationAndLocationCode =  {'station': '232', 'nearCityStation': '129', 'longitude': '-126/49/58', 'latitude':'36/42/0', 'midTermForecastCityCode':'11C20101'}; break;
		
		// Boryeong
		case '44180': 
		stationAndLocationCode =  {'station': '235', 'nearCityStation': '129', 'longitude': '-126/46/58', 'latitude':'36/19/58', 'midTermForecastCityCode':'11C20101'}; break;
		// 서천군
		case '44770':
		stationAndLocationCode =  {'station': '235', 'nearCityStation': '129', 'longitude': '-126/40/58', 'latitude':'36/4/1', 'midTermForecastCityCode':'11C20101'}; break;
		// 청양군
		case '44790':
		stationAndLocationCode =  {'station': '235', 'nearCityStation': '129', 'longitude': '-126/49/1', 'latitude':'36/27/0', 'midTermForecastCityCode':'11C20101'}; break;
		
		// Seosan
		case '44210': 
		stationAndLocationCode =  {'station': '129', 'longitude': '-126/26/59', 'latitude':'36/46/1', 'midTermForecastCityCode':'11C20101'}; break;
		// 태안군
		case '44825':
		stationAndLocationCode =  {'station': '129', 'longitude': '-126/16/58', 'latitude':'36/45/0', 'midTermForecastCityCode':'11C20101'}; break;
		// 홍성군
		case '44800':
		stationAndLocationCode =  {'station': '129', 'longitude': '-126/40/1', 'latitude':'36/35/59', 'midTermForecastCityCode':'11C20101'}; break;
		// 당진군
		case '44830':
		stationAndLocationCode =  {'station': '129', 'longitude': '-126/37/1', 'latitude':'36/54/0', 'midTermForecastCityCode':'11C20101'}; break;
		// Buyeo
		case '44760':
		stationAndLocationCode =  {'station': '129', 'longitude': '-126/54/0', 'latitude':'36/16/1', 'midTermForecastCityCode':'11C20101'}; break;
		
		// Daejeon
		case '44250': 
		stationAndLocationCode =  {'station': '133', 'longitude': '-127/25/1', 'latitude':'36/19/1', 'midTermForecastCityCode':'11C20401'}; break;
		// 공주시
		case '44150':
		stationAndLocationCode =  {'station': '133', 'longitude': '-127/7/58', 'latitude':'36/27/0', 'midTermForecastCityCode':'11C20401'}; break;
		// 연기군
		case '44730':
		stationAndLocationCode =  {'station': '133', 'longitude': '-127/16/58', 'latitude':'36/37/58', 'midTermForecastCityCode':'11C20401'}; break;
		// Geumsan
		case '44710': 
		stationAndLocationCode =  {'station': '133', 'longitude': '-127/28/1', 'latitude':'36/4/58', 'midTermForecastCityCode':'11C20401'}; break;
		// 논산시
		case '44230':
		stationAndLocationCode =  {'station': '133', 'longitude': '-127/11/59', 'latitude':'36/12/0', 'midTermForecastCityCode':'11C20401'}; break;
		
		// Jeonju
		case '45111': case '45113':
		stationAndLocationCode =  {'station': '146', 'longitude': '-127/7/58', 'latitude':'35/47/59', 'midTermForecastCityCode':'11F10201'}; break;
		// 완주군
		case '45710':
		stationAndLocationCode =  {'station': '146', 'longitude': '-127/9/0', 'latitude':'35/55/58', 'midTermForecastCityCode':'11F10201'}; break;
		// 익산시
		case '45140':
		stationAndLocationCode =  {'station': '146', 'longitude': '-126/56/59', 'latitude':'35/55/58', 'midTermForecastCityCode':'11F10201'}; break;
		// Jangsu
		case '45740': 
		stationAndLocationCode =  {'station': '146', 'longitude': '-127/31/1', 'latitude':'35/39/0', 'midTermForecastCityCode':'11F10201'}; break;
		// 무주군
		case '45730':
		stationAndLocationCode =  {'station': '146', 'longitude': '-127/39/0', 'latitude':'36/0/0', 'midTermForecastCityCode':'11F10201'}; break;
		// 진안군
		case '45720':
		stationAndLocationCode =  {'station': '146', 'longitude': '-127/25/1', 'latitude':'35/46/58', 'midTermForecastCityCode':'11F10201'}; break;
		// Imsil
		case '45750': 
		stationAndLocationCode =  {'station': '146', 'longitude': '-127/16/58', 'latitude':'35/37/1', 'midTermForecastCityCode':'11F10201'}; break;
		
		// Gunsan
		case '45130': 
		stationAndLocationCode =  {'station': '140', 'nearCityStation': '146', 'longitude': '-126/41/59', 'latitude':'35/58/58', 'midTermForecastCityCode':'11F10201'}; break;
		// 김제시
		case '45210':
		stationAndLocationCode =  {'station': '140', 'nearCityStation': '146', 'longitude': '-126/56/59', 'latitude':'35/17/59', 'midTermForecastCityCode':'11F10201'}; break;
		
		// Jeongeup
		case '45180': 
		stationAndLocationCode =  {'station': '245', 'nearCityStation': '146', 'longitude': '-126/52/1', 'latitude':'35/34/1', 'midTermForecastCityCode':'11F10201'}; break;
		// Buan
		case '45800': 
		stationAndLocationCode =  {'station': '245', 'nearCityStation': '146', 'longitude': '-126/43/58', 'latitude':'35/43/58', 'midTermForecastCityCode':'11F10201'}; break;
		
		// Namwon
		case '45190': 
		stationAndLocationCode =  {'station': '247', 'nearCityStation': '146', 'longitude': '-127/22/1', 'latitude':'35/25/1', 'midTermForecastCityCode':'11F10201'}; break;
		// Sunchang
		case '45770': 
		stationAndLocationCode =  {'station': '247', 'nearCityStation': '156', 'longitude': '-127/7/58', 'latitude':'35/24/0', 'midTermForecastCityCode':'11F10201'}; break;
		
		// Gochang
		case '45790': 
		stationAndLocationCode =  {'station': '172', 'nearCityStation': '156', 'longitude': '-126/41/59', 'latitude':'35/25/1', 'midTermForecastCityCode':'11F10201'}; break;
		
		// Mokpo
		case '46110':
		if (Point.z3 == '4691036000') { // Heuksando
			stationAndLocationCode =  {'station': '169', 'longitude': '-125/25/10', 'latitude':' 34/40/14', 'midTermForecastCityCode':'21F20801'};
		} else {
			stationAndLocationCode =  {'station': '165', 'longitude': '-126/22/58', 'latitude':'34/46/58', 'midTermForecastCityCode':'21F20801'};
		}
		break;
		// 무안군
		case '46840':
		stationAndLocationCode =  {'station': '165', 'longitude': '-126/28/58', 'latitude':'34/58/58', 'midTermForecastCityCode':'21F20801'}; break;
		// 신안군
		case '46910':
		stationAndLocationCode =  {'station': '165', 'longitude': '-126/7/58', 'latitude':'34/15/0', 'midTermForecastCityCode':'21F20801'}; break;
		// 영암군
		case '46830':
		stationAndLocationCode =  {'station': '165', 'longitude': '-126/41/59', 'latitude':'34/46/58', 'midTermForecastCityCode':'21F20801'}; break;
		// Yeonggwang
		case '46870': 
		stationAndLocationCode =  {'station': '165', 'longitude': '-126/31/1', 'latitude':'35/16/1', 'midTermForecastCityCode':'21F20801'}; break;
		// 함평군
		case '46860':
		stationAndLocationCode =  {'station': '165', 'longitude': '-126/31/1', 'latitude':'35/2/59', 'midTermForecastCityCode':'21F20801'}; break;
		
		// Yoesu
		case '46130':
		stationAndLocationCode =  {'station': '168', 'longitude': '-127/43/58', 'latitude':'34/43/58', 'midTermForecastCityCode':'11F20401'}; break;
		// 광양시
		case '46230':
		stationAndLocationCode =  {'station': '168', 'longitude': '-127/40/58', 'latitude':'34/58/1', 'midTermForecastCityCode':'11F20401'}; break;
		// Goheung
		case '46770': 
		stationAndLocationCode =  {'station': '168', 'longitude': '-127/16/58', 'latitude':'34/5/59', 'midTermForecastCityCode':'11F20401'}; break;
		// 보성군
		case '46780':
		stationAndLocationCode =  {'station': '168', 'longitude': '-127/4/58', 'latitude':'34/16/1', 'midTermForecastCityCode':'11F20401'}; break;
		// Namhae
		case '48840': 
		stationAndLocationCode =  {'station': '168', 'longitude': '-127/54/0', 'latitude':'34/46/1', 'midTermForecastCityCode':'11F20401'}; break;
		
		// Suncheon
		case '46150': 
		stationAndLocationCode =  {'station': '174', 'nearCityStation': '168', 'longitude': '-127/28/58', 'latitude':'34/55/58', 'midTermForecastCityCode':'11F20401'}; break;
		// 곡성군
		case '46720':
		stationAndLocationCode =  {'station': '174', 'nearCityStation': '168', 'longitude': '-127/16/58', 'latitude':'35/16/58', 'midTermForecastCityCode':'11F20401'}; break;
		// 구례군
		case '46730':
		stationAndLocationCode =  {'station': '174', 'nearCityStation': '168', 'longitude': '-127/26/59', 'latitude':'35/12/0', 'midTermForecastCityCode':'11F20401'}; break;
		
		// 화순군
		case '46790': 
		stationAndLocationCode =  {'station': '156', 'longitude': '-126/58/58', 'latitude':'35/2/59', 'midTermForecastCityCode':'11F20501'}; break;
		// 나주시
		case '46170':
		stationAndLocationCode =  {'station': '156', 'longitude': '-126/43/1', 'latitude':'35/1/58', 'midTermForecastCityCode':'11F20501'}; break;
		// 담양군
		case '46710':
		stationAndLocationCode =  {'station': '156', 'longitude': '-126/58/1', 'latitude':'35/19/1', 'midTermForecastCityCode':'11F20501'}; break;
		// 장성군
		case '46880':
		stationAndLocationCode =  {'station': '156', 'longitude': '-126/45/0', 'latitude':'35/16/58', 'midTermForecastCityCode':'11F20501'}; break;
		
		// Wando
		case '46890': 
		stationAndLocationCode =  {'station': '170', 'nearCityStation': '165', 'longitude': '-126/45/0', 'latitude':'34/17/59', 'midTermForecastCityCode':'21F20801'}; break;
		// Haenam
		case '46820': 
		stationAndLocationCode =  {'station': '170', 'nearCityStation': '165', 'longitude': '-126/35/59', 'latitude':'34/34/1', 'midTermForecastCityCode':'21F20801'}; break;
		// Jangheung
		case '46800':  
		stationAndLocationCode =  {'station': '170', 'nearCityStation': '165', 'longitude': '-126/54/0', 'latitude':'34/40/58', 'midTermForecastCityCode':'21F20801'}; break;
		// 강진군
		case '46810':
		stationAndLocationCode =  {'station': '170', 'nearCityStation': '165', 'longitude': '-126/46/1', 'latitude':'34/37/58', 'midTermForecastCityCode':'21F20801'}; break;
		
		// Jindo
		case '46900': 
		stationAndLocationCode =  {'station': '175', 'nearCityStation': '165', 'longitude': '-126/16/1', 'latitude':'34/28/58', 'midTermForecastCityCode':'21F20801'}; break;
		
		// Pohang
		case '47111': case '47113':
		stationAndLocationCode =  {'station': '138', 'longitude': '-129/22/1', 'latitude':'36/1/58', 'midTermForecastCityCode':'11H10701'}; break;
		// 경주시
		case '47130':
		stationAndLocationCode =  {'station': '138', 'longitude': '-129/11/59', 'latitude':'35/49/58', 'midTermForecastCityCode':'11H10701'}; break;
		// Yeongdeok
		case '47770': 
		stationAndLocationCode =  {'station': '138', 'longitude': '-129/22/1', 'latitude':'36/24/0', 'midTermForecastCityCode':'11H10701'}; break;
		
		// Gumi
    case '47190':
		stationAndLocationCode =  {'station': '279', 'nearCityStation': '143', 'longitude': '-128/19/58', 'latitude':'36/5/59', 'midTermForecastCityCode':'11H10701'}; break;
		// 군위군
		case '47720':
		stationAndLocationCode =  {'station': '279', 'nearCityStation': '143', 'longitude': '-128/34/1', 'latitude':'36/13/1', 'midTermForecastCityCode':'11H10701'}; break;
		// 김천시
		case '47150':
		stationAndLocationCode =  {'station': '279', 'nearCityStation': '143', 'longitude': '-128/7/1', 'latitude':'36/7/1', 'midTermForecastCityCode':'11H10701'}; break;
		// 성주군
		case '47840':
		stationAndLocationCode =  {'station': '279', 'nearCityStation': '143', 'longitude': '-128/16/58', 'latitude':'35/54/0', 'midTermForecastCityCode':'11H10701'}; break;
		
		// Andong
		case '47170': 
		stationAndLocationCode =  {'station': '136', 'longitude': '-128/43/1', 'latitude':'36/34/1', 'midTermForecastCityCode':'11H10501'}; break;
		// Yeongju
		case '47210': 
		stationAndLocationCode =  {'station': '136', 'longitude': '-128/37/1', 'latitude':'36/49/1', 'midTermForecastCityCode':'11H10501'}; break;
		// Uiseong
		case '47730':
		stationAndLocationCode =  {'station': '136', 'longitude': '-128/40/58', 'latitude':'36/20/59', 'midTermForecastCityCode':'11H10501'}; break;
		// 청송군
		case '47750':
		stationAndLocationCode =  {'station': '136', 'longitude': '-129/3/0', 'latitude':'36/25/1', 'midTermForecastCityCode':'11H10501'}; break;
		// Bongwhoa
		case '47920': 
		stationAndLocationCode =  {'station': '136', 'longitude': '-128/45/0', 'latitude':'36/52/58', 'midTermForecastCityCode':'11H10501'}; break;
		// 영양군
		case '47760':
		stationAndLocationCode =  {'station': '136', 'longitude': '-129/6/0', 'latitude':'36/40/1', 'midTermForecastCityCode':'11H10501'}; break;
		// 예천군
		case '47900':
		stationAndLocationCode =  {'station': '136', 'longitude': '-128/26/59', 'latitude':'36/28/58', 'midTermForecastCityCode':'11H10501'}; break;
		
		// Sangju
		case '47250': 
		stationAndLocationCode =  {'station': '137', 'nearCityStation': '136', 'longitude': '-128/8/59', 'latitude':'36/25/1', 'midTermForecastCityCode':'11H10501'}; break;
		// Mungyeong
		case '47280': 
		stationAndLocationCode =  {'station': '137', 'nearCityStation': '136', 'longitude': '-128/10/58', 'latitude':'36/34/58', 'midTermForecastCityCode':'11H10501'}; break;
		
		// Daegu
    case '47850': 
		stationAndLocationCode =  {'station': '143', 'longitude': '-128/34/58', 'latitude':'35/50/59', 'midTermForecastCityCode':'11H10701'}; break;
		// 경산시
		case '47290':
		stationAndLocationCode =  {'station': '143', 'longitude': '-128/43/58', 'latitude':'35/49/1', 'midTermForecastCityCode':'11H10701'}; break;
		// 청도군
		case '47820':
		stationAndLocationCode =  {'station': '143', 'longitude': '-128/46/1', 'latitude':'35/39/0', 'midTermForecastCityCode':'11H10701'}; break;
		// 고령군
		case '47830':
		stationAndLocationCode =  {'station': '143', 'longitude': '-128/16/1', 'latitude':'35/43/1', 'midTermForecastCityCode':'11H10701'}; break;
		// Yeongcheon
		case '47230': 
		stationAndLocationCode =  {'station': '143', 'longitude': '-128/55/58', 'latitude':'35/57/0', 'midTermForecastCityCode':'11H10701'}; break;
		
		// Uljin
		case '47930': 
		stationAndLocationCode =  {'station': '130', 'longitude': '-129/22/58', 'latitude':'36/58/58', 'midTermForecastCityCode':'11H10501'}; break;
		
		// Ulleungdo
		case '47940':
		if (Point.z3 == '4794033000') { // 독도
			stationAndLocationCode =  {'station': '115', 'longitude': '-131/52/10', 'latitude':'37/14/27', 'midTermForecastCityCode':'11H10501'};
		} else {
			stationAndLocationCode =  {'station': '115', 'longitude': '-130/52/1', 'latitude':'37/34/1', 'midTermForecastCityCode':'11H10501'};
		}
		break;
		
		// Changwon
		case '48110': 
		stationAndLocationCode =  {'station': '155', 'longitude': '-128/40/1', 'latitude':'35/13/58', 'midTermForecastCityCode':'11H20301'}; break;
		// Masan
		case '48160': 
		stationAndLocationCode =  {'station': '155', 'longitude': '-128/34/1', 'latitude':'35/10/58', 'midTermForecastCityCode':'11H20301'}; break;
		// 함안군
		case '48730': 
		stationAndLocationCode =  {'station': '155', 'longitude': '-128/23/59', 'latitude':'35/16/1', 'midTermForecastCityCode':'11H20301'}; break;
		// 진해구
		case '48190':
		stationAndLocationCode =  {'station': '155', 'longitude': '-128/38/59', 'latitude':'35/7/58', 'midTermForecastCityCode':'11H20301'}; break;
		// 창녕군
		case '48740':
		stationAndLocationCode =  {'station': '155', 'longitude': '-128/30/0', 'latitude':'35/31/58', 'midTermForecastCityCode':'11H20301'}; break;
		
		// Jinju
		case '48170': 
		stationAndLocationCode =  {'station': '192', 'longitude': '-128/4/1', 'latitude':'35/10/58', 'midTermForecastCityCode':'11H20301'}; break;
		// 사천시
		case '48240':
		stationAndLocationCode =  {'station': '192', 'longitude': '-128/4/1', 'latitude':'34/50/59', 'midTermForecastCityCode':'11H20301'}; break;
		// Sancheong
		case '48860': 
		stationAndLocationCode =  {'station': '192', 'longitude': '-127/52/58', 'latitude':'35/24/0', 'midTermForecastCityCode':'11H20301'}; break;
		// 하동군
		case '48850':
		stationAndLocationCode =  {'station': '192', 'longitude': '-127/45/0', 'latitude':'35/4/1', 'midTermForecastCityCode':'11H20301'}; break;
		
		// Tongyeong
		case '48220': 
		stationAndLocationCode =  {'station': '162', 'longitude': '-128/25/58', 'latitude':'34/46/1', 'midTermForecastCityCode':'11H20301'}; break;
		// 경상남도 고성군
		case '48820':
		stationAndLocationCode =  {'station': '162', 'longitude': '-128/19/58', 'latitude':'34/54/0', 'midTermForecastCityCode':'11H20301'}; break;
		
		// Geoje
		case '48310': 
		stationAndLocationCode =  {'station': '294', 'nearCityStation': '162', 'longitude': '-128/37/1', 'latitude':'34/49/1', 'midTermForecastCityCode':'11H20301'}; break;
		
		// Geochang
		case '48880': 
		stationAndLocationCode =  {'station': '284', 'longitude': '-127/54/57', 'latitude':'35/40/1', 'midTermForecastCityCode':'11H10701'}; break;
		// 함양군
		case '48870':
		stationAndLocationCode =  {'station': '284', 'longitude': '-127/43/1', 'latitude':'35/50/59', 'midTermForecastCityCode':'11H10701'}; break;
		// Hapcheon
		case '48890': 
		stationAndLocationCode =  {'station': '284', 'longitude': '-128/10/1', 'latitude':'35/32/59', 'midTermForecastCityCode':'11H10701'}; break;
		// 의령군
		case '48720':
		stationAndLocationCode =  {'station': '284', 'longitude': '-128/16/1', 'latitude':'35/19/1', 'midTermForecastCityCode':'11H20301'}; break;
		
		// Jeju
		case '50110': 
		stationAndLocationCode =  {'station': '184', 'longitude': '-126/31/58', 'latitude':'33/30/0', 'midTermForecastCityCode':'11G00201'}; break;
		
		// Seogwipo
		case '50130': 
		stationAndLocationCode =  {'station': '189', 'longitude': '-126/34/1', 'latitude':'33/15/0', 'midTermForecastCityCode':'11G00401'}; break;
	}
	stationAndLocationCode.AWSstation = AWSstationCode[currentPoint];
	return stationAndLocationCode;
}

function getForecastWeatherIcon(w, t) {
	if (w == 'clear') {
		switch (t) {
			case '21': case '24': case '3':
				return 'sunny-moon';
			default:
				return 'sunny';
		}
	} else if ((w == 'mostly cloudy') || (w == 'partly cloudy')) {
		switch (t) {
			case '21': case '24': case '3':
				return w + '-moon';
			default:
				return w;
		}
	} else if (w == 'snow/rain') {
		return 'rain and snow';
	} else {
		return w;
	}
}

// Default functions for behavior when Dashboard is activated/deactivated/removed/focus/blur
if (window.widget) {
    widget.onshow = onshow;
    widget.onhide = onhide;
    widget.onremove = onremove;
}

function onshow() {
    getWeather();
}

function onhide() {
	if (weatherRequest) {
		weatherRequest.abort();
		endFetchingAnimation();
	}
	if (weatherDataRequest) weatherDataRequest.abort();
	if (shortTermForecastRequest) shortTermForecastRequest.abort();
	if (midTermForecastRequest) midTermForecastRequest.abort();
	endRainAlertAnimation();
	if (isSnowing) {
		snowStorm.toggleSnow();
		isSnowing = false;
	}
	document.getElementById('stars').style.display = 'none';
}

// When the widget is removed from the Dashboard, delete all the preferences
function onremove() {
	widget.setPreferenceForKey(null, createkey('point'));
	widget.setPreferenceForKey(null, createkey('point2nd'));
	widget.setPreferenceForKey(null, createkey('pointindex'));
	widget.setPreferenceForKey(null, createkey('fahrenheit'));
	widget.setPreferenceForKey(null, createkey('iconindex'));
	widget.setPreferenceForKey(null, createkey('windiconindex'));
	widget.setPreferenceForKey(null, createkey('snoweffect'));
	widget.setPreferenceForKey(null, createkey('skinkey'));
}

function showPrefs() {		
	if (actualSnowFallingEffect) {
		document.getElementById('snowbox').checked = true;
	} else {
		document.getElementById('snowbox').checked = false;
	}
	
	if (isSnowing) {
		snowStorm.toggleSnow();
		isSnowing = false;
	}
	
	if (window.widget) {
		widget.prepareForTransition('ToBack');
	}
       
	document.getElementById('front').style.display='none';
	document.getElementById('back').style.display='block';

	if (window.widget) {
		setTimeout('widget.performTransition();', 100);
	}
}

function hidePrefs() {	
	if (window.widget) {
		widget.prepareForTransition('ToFront');
	}
    
	document.getElementById('back').style.display='none';
	document.getElementById('front').style.display='block';

	if (window.widget) {
		setTimeout('widget.performTransition();', 100);
	}
		
	getWeather();
	
	originalAngle = windDirection + sweepbackMax;
	var angle = originalAngle / 180 * Math.PI; // all angles computed in radians
	drawWindVane(angle);
	makeWindVaneSproing();
}

function populateAreaNamePopup() {
	var areaNameSelect = document.getElementById('area-name-popup');
	var oFragment = document.createDocumentFragment();
	var myPointTownList = localizedTownNames[myPoint.z2];
	var areaName, i, j, element;
	for (i = 0, j = myPointTownList.length; i < j; ++i) {
		if (myPointTownList[i].code == myPoint.z3) {
			element = document.createElement('option');
			element.text = myPointTownList[i].name;
			element.value = myPoint.z3;
			oFragment.appendChild(element);
			areaName = myPointTownList[i].name;
			break;
		}
	}
	
	var myPoint2ndTownList = localizedTownNames[myPoint2nd.z2];
	for (i = 0, j = myPoint2ndTownList.length; i < j; ++i) {
		if (myPoint2ndTownList[i].code == myPoint2nd.z3) {
			element = document.createElement('option');
			element.text = myPoint2ndTownList[i].name;
			element.value = myPoint2nd.z3;
			oFragment.appendChild(element);
			break;
		}
	}

	areaNameSelect.appendChild(oFragment);
  areaNameSelect.options[currentPointIndex].selected = true;
}

function areachanged(select) {
	var provinceSelect = document.getElementById('province-popup');
	var citySelect = document.getElementById('city-popup');
	var townSelect = document.getElementById('town-popup');
	var nodelist, numOfList, i;
	
	currentPointIndex = select.selectedIndex;
	currentPoint = (currentPointIndex === 0) ? myPoint : myPoint2nd;
	widget.setPreferenceForKey(currentPointIndex, createkey('pointindex'));
		
	if (currentPoint.z1 != provinceSelect.options[provinceSelect.selectedIndex].value) {
		nodelist = provinceSelect.childNodes;
		numOfList = nodelist.length;
		for (i = numOfList-1; i > -1; --i) {
			this_node = nodelist[i];
			provinceSelect.removeChild(this_node);
		}
		populateProvinceSelect(currentPoint);
	} else if (currentPoint.z2 != citySelect.options[citySelect.selectedIndex].value) {
		nodelist = citySelect.childNodes;
		numOfList = nodelist.length;
		for (i = numOfList-1; i > -1; --i) {
			this_node = nodelist[i];
			citySelect.removeChild(this_node);
		}
		populateCitySelect(currentPoint);
	} else {
		nodelist = townSelect.childNodes;
		numOfList = nodelist.length;
		for (i = numOfList-1; i > -1; --i) {
			this_node = nodelist[i];
			townSelect.removeChild(this_node);
		}
		populateTownSelect(currentPoint);
	}
	midTermForecastLastUpdated = undefined;
	getWeather();
}

function populateProvinceSelect(point) {
	var provinceSelect = document.getElementById('province-popup');
	var oFragment = document.createDocumentFragment();
	var doc = document;
	
	for (var i = 0, j = localizedProvinceNames.length; i < j; ++i) {
		var element = doc.createElement('option');
		element.text = localizedProvinceNames[i].name;
		element.value = localizedProvinceNames[i].code;
		if (localizedProvinceNames[i].code == point.z1) {
		  element.selected = true;
	  }
		oFragment.appendChild(element);
	}
	provinceSelect.appendChild(oFragment);
	
	populateCitySelect(point);
}

function provincechanged(select) {
	if (currentPoint.z1 != select.options[select.selectedIndex].value) {	
		var citySelect = document.getElementById('city-popup');
		var nodelist = citySelect.childNodes;
		var numOfList = nodelist.length;
		for (var i = numOfList-1; i > -1; --i) {
			this_node = nodelist[i];
			citySelect.removeChild(this_node);
		}
		
		var townSelect = document.getElementById('town-popup');
		nodelist = townSelect.childNodes;
		numOfList = nodelist.length;
		for (i = numOfList-1; i > -1; --i) {
			this_node = nodelist[i];
			townSelect.removeChild(this_node);
		}
		
		populateCitySelect();
	}
}

function populateCitySelect(point) {
	var citySelect = document.getElementById('city-popup');
	var provinceSelect = document.getElementById('province-popup');
	var provinceValue = provinceSelect.options[provinceSelect.selectedIndex].value;
	var i, j;
	
	if (point !== undefined) {
		var nodelist = citySelect.childNodes;
		var numOfList = nodelist.length;
		for (i = numOfList-1; i > -1; --i) {
			this_node = nodelist[i];
			citySelect.removeChild(this_node);
		}
	}
	
	var oFragment = document.createDocumentFragment();
	var cityList = localizedCityNames[provinceValue];
	cityList.sort(sortZipText);
	
	var doc = document;
	for (i = 0, j = cityList.length; i < j; ++i) {
		var element = doc.createElement('option');
		element.text = cityList[i].name;
		element.value = cityList[i].code;
		if ((point !== undefined) && (cityList[i].code == point.z2)) {
		  element.selected = true;
	  }
		oFragment.appendChild(element);
	}
	citySelect.appendChild(oFragment);
	if (citySelect.selectedIndex == -1) {
	  citySelect.selectedIndex = 0;
  }
	
	populateTownSelect(point);
}

function citychanged(select) {
	if (currentPoint.z2 != select.options[select.selectedIndex].value) {	
		var townSelect = document.getElementById('town-popup');
		var nodelist = townSelect.childNodes;
		var numOfList = nodelist.length;
		for (var i = numOfList-1; i > -1; --i) {
			this_node = nodelist[i];
			townSelect.removeChild(this_node);
		}

		populateTownSelect();
	}
}

function populateTownSelect(point) {
	var townSelect = document.getElementById('town-popup');
	var citySelect = document.getElementById('city-popup');
	var cityValue = citySelect.options[citySelect.selectedIndex].value;
	var i, j;
	
	if (point !== undefined) {
		var nodelist = townSelect.childNodes;
		var numOfList = nodelist.length;
		for (i = numOfList-1; i > -1; --i) {
			this_node = nodelist[i];
			townSelect.removeChild(this_node);
		}
	}
	
	var oFragment = document.createDocumentFragment();
	var townList = localizedTownNames[cityValue];
	townList.sort(sortZipText);
	
	for (i = 0, j = townList.length; i < j; ++i) {
		var element = document.createElement('option');
		element.text = townList[i].name;
		element.value = townList[i].code;
		if ((point !== undefined) && (townList[i].code == point.z3)) {
		  element.selected = true;
	  }
		oFragment.appendChild(element);
	}
	townSelect.appendChild(oFragment);
	if (townSelect.selectedIndex == -1) {
	  townSelect.selectedIndex = 0;
  }
	
	townchanged(townSelect);
}

function townchanged(select) {
	var selectedPoint = select.options[select.selectedIndex].value;
	var areaNameSelect = document.getElementById('area-name-popup');
	var nodelist, i;
	
	if (currentPointIndex === 0) {
		if (selectedPoint != myPoint.z3) {
			myPoint = new Point(selectedPoint);
			widget.setPreferenceForKey(myPoint.z3, createkey('point'));
			stationAndLocationCode = getStationAndLocationCode(myPoint);
			currentPoint = myPoint;
			nodelist = areaNameSelect.childNodes;
			for (i = 1; i > -1; --i) {
				this_node = nodelist[i];
				areaNameSelect.removeChild(this_node);
			}
			populateAreaNamePopup();
			areaNameSelect.options[0].selected = true;
		}
	} else if (selectedPoint != myPoint2nd.z3) {
			myPoint2nd = new Point(selectedPoint);
			widget.setPreferenceForKey(myPoint2nd.z3, createkey('point2nd'));
			stationAndLocationCode2nd = getStationAndLocationCode(myPoint2nd);
			currentPoint = myPoint2nd;
			nodelist = areaNameSelect.childNodes;
			for (i = 1; i > -1; --i) {
				this_node = nodelist[i];
				areaNameSelect.removeChild(this_node);
			}
			populateAreaNamePopup();
			areaNameSelect.options[1].selected = true;
	}
	midTermForecastLastUpdated = undefined;
}

function sortZipText(a,b) {
	if (a.name > b.name) return 1;
	else if ( a.name < b.name) return -1;
	else return 0;
}

function Point(zone) {
	this.z1 = zone.substring(0,2);
	this.z2 = zone.substring(0,5);
	this.z3 = zone;
}

function populateTempUnits() {
	var select = document.getElementById ('units-popup');
	var element;
	
	element = document.createElement ('option');
	element.text = 'ºC';
	select.appendChild(element);
	element = document.createElement ('option');
	element.text = 'ºF';
	select.appendChild(element);
	
	if (actualFahrenheit) {
		select.options[1].selected = true; // fahrenheit selected
	} else {
		select.options[0].selected = true; // celsius selected
	}
}

function unitschanged(select) {
	var isF = select.selectedIndex == 1;
	
	if (isF != actualFahrenheit) {
		actualFahrenheit = isF;
		var decimalPoint;

		if (window.widget) {
			widget.setPreferenceForKey(actualFahrenheit ? true : false, createkey('fahrenheit'));
		}

		var temperature = parseFloat(weatherData[7]);
		if (!isNaN(temperature)) {
      if (temperature % 1 === 0) {
        decimalPoint = '.0';
      } else {
        decimalPoint = '';
      }
				
      document.getElementById('temperature').innerText = convertToFahrenheit(weatherData[7]) + decimalPoint + 'º';
    }
    midTermForecastLastUpdated = undefined; // to recalculate the hi-low temperature
	}
}

function populateIconSelect() {
	var select = document.getElementById ('icon-popup');
	var element;
	
	for (var i = 0, j = weatherIconNames.noOfIcons; i < j; ++i) {
		element = document.createElement ('option');
		element.text = weatherIconNames[i];
		select.appendChild(element);
	}
}

function iconchanged(select) {
	if (actualIconIndex != select.selectedIndex) {
		actualIconIndex = select.selectedIndex;
    document.getElementById('icon-popup').options[actualIconIndex].selected = true;
		widget.setPreferenceForKey(actualIconIndex, createkey('iconindex'));
		iconName = weatherIconNames[actualIconIndex];

		showWeatherIcon();
		showForecastWeatherIcons();
		updateMidTermForecastIcon();
	}
}

function snowChecked() {
	if (document.getElementById('snowbox').checked) {
		actualSnowFallingEffect = true;
	} else {
		actualSnowFallingEffect = false;
		if (isSnowing) snowStorm.stop();
		isSnowing = false;
	}
	widget.setPreferenceForKey(actualSnowFallingEffect, createkey('snoweffect'));
}

function skinChanged(event) {
	// Get new skin
	var sender = event.target;
	if (sender.tagName == 'IMG') {
		newSkin = sender.id.replace('ButtonImg', '');
		// Set new skin
		setSkin(newSkin);		
	}
}

function setSkin(newSkin) {
	// Get old skin
	var oldSkin;
	oldSkin = skin;
	
	// Update preference
	skin = newSkin;
	widget.setPreferenceForKey(skin, createkey('skinkey'));
	
	var doc = document;
	// Update skin buttons
	doc.getElementById(oldSkin + 'ButtonImg').src = 'images/buttons/' + oldSkin + '.png';
	doc.getElementById(newSkin + 'ButtonImg').src = 'images/buttons/' + newSkin + '-checked.png';
	
	// Update border image
	if (isTiger) {
		doc.getElementById('top').style.backgroundImage = 'url(images/backgrounds/tiger/' + skin + '/top.png)';
		doc.getElementById('right').style.backgroundImage = 'url(images/backgrounds/tiger/' + skin + '/right.png)';
		doc.getElementById('bottom').style.backgroundImage = 'url(images/backgrounds/tiger/' + skin + '/bottom.png)';
		doc.getElementById('left').style.backgroundImage = 'url(images/backgrounds/tiger/' + skin + '/left.png)';
	} else {
		doc.getElementById('widget-border').style.backgroundImage = 'url(images/backgrounds/' + skin + '.png)';
	}
}

function convertToFahrenheit(num) {
	if ((actualFahrenheit) && (num != '-')) {
		return Math.round ((num * 9 / 5) +32);
	} else {
		return num;
	}
}

function drawCheckFailure(message) {
  //fill in listings section
  var warning = document.getElementById('notice');
  warning.innerText = message;
  warning.style.display = 'block';
  
  if (window.widget) {
    setTimeout(function() {
      document.getElementById('notice').style.display = 'none';
    }, 10000);
  }
}

function goHome() {
	widget.openURL('http://appletree.or.kr/widgets/WeatherOutside/');
}

// Download version.txt from website and check version numbers
function checkForNewVersion() {
	var versionURL = 'http://appletree.or.kr/widgets/WeatherOutside/version.txt';
	
	if (versionRequest)
		versionRequest.abort();

	versionRequest = new XMLHttpRequest();
	versionRequest.onreadystatechange = getVersionNumber;
	versionRequest.open('GET', versionURL, true);
	versionRequest.send(null);
}

// If new version is available, display link and notice
function getVersionNumber() {
	try {
		if (versionRequest.readyState == 4 && versionRequest.status == 200) {
			if (versionRequest.responseText !== versionNum) {
				document.getElementById('newMessage').innerText = getLocalizedString('NewVersion');
				document.getElementById('newVersion').style.display = 'block';
				setTimeout(function() {
					document.getElementById('newVersion').style.display = 'none';
				}, 9000);
			}

			versionRequest = null;
		} else if (versionRequest.readyState == 4 && versionRequest.status != 200) {
      versionRequest.abort();
      versionRequest = null;
		}
	}
	catch(exception) {
		//drawCheckFailure(getLocalizedString('Failed') + exception);
	}
}

// Flipping animation: From Apple Developer Website
var flipShown = false;
var animation = {duration:0, starttime:0, to:1.0, now:0.0, from:0.0, firstElement:null, timer:null};

function mousemove() {
	if (!flipShown) {
		if (animation.timer !== null) {
			clearInterval (animation.timer);
			animation.timer  = null;
		}
    var now = new Date();
		var starttime = now.getTime() - 13;
 
		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById ('info');
		animation.timer = setInterval (animate, 13);
		animation.from = animation.now;
		animation.to = 1.0;
		animate();
		flipShown = true;
	}
}

function mouseexit() {
	if (flipShown) {
		// fade in the info button
		if (animation.timer !== null) {
			clearInterval (animation.timer);
			animation.timer  = null;
		}
		var now = new Date();
		var starttime = now.getTime() - 13;

		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById ('info');
		animation.timer = setInterval (animate, 13);
		animation.from = animation.now;
		animation.to = 0.0;
		animate();
		flipShown = false;
	}
}

function animate() {
	var T;
	var ease;
	var now = new Date();
	var time = now.getTime();
   

	T = limit_3(time-animation.starttime, 0, animation.duration);

	if (T >= animation.duration) {
		clearInterval (animation.timer);
		animation.timer = null;
		animation.now = animation.to;
	} else {
		ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
		animation.now = computeNextFloat (animation.from, animation.to, ease);
	}

	animation.firstElement.style.opacity = animation.now;
}

function limit_3(a, b, c) {
	return a < b ? b : (a > c ? c : a);
}

function computeNextFloat(from, to, ease) {
	return from + (to - from) * ease;
}

// fatching animation from the Apple's Weather widget
var fetchingAnimation = {timer: null, alphaNow:1.0, up:false, duration:1000, start:null};

function fetchAnimationTimer() {
	var now = new Date();
	var time = now.getTime();
	var fraction;
	
	if (fetchingAnimation.up) {
		fraction = (time - fetchingAnimation.start) / fetchingAnimation.duration;
	} else {
		fraction = 1.0 - (time - fetchingAnimation.start) / fetchingAnimation.duration;
	}
	
	fetchingAnimation.alphaNow = fraction;
	
	if (fetchingAnimation.alphaNow > 1.0) {
		fetchingAnimation.start = time;
		fetchingAnimation.alphaNow = 1.0;
		fetchingAnimation.up = false;
	} else if (fetchingAnimation.alphaNow < 0.0) {
		fetchingAnimation.start = time;
		fetchingAnimation.alphaNow = 0.0;
		fetchingAnimation.up = true;
	}
	
	var sine = Math.sin((Math.PI/2.0) * fetchingAnimation.alphaNow);
	fetchingAnimation.alphaNow = sine * sine;
	document.getElementById('temperature').style.opacity = fetchingAnimation.alphaNow;
}

function startFetchingAnimation() {
	if (fetchingAnimation.timer !== null) {
		clearInterval (fetchingAnimation.timer);
		fetchingAnimation.timer = null;
	}
	var now = new Date();
	fetchingAnimation.up = false;
	fetchingAnimation.alphaNow = 1.0;
	fetchingAnimation.start = now.getTime() - 60;
	fetchingAnimation.timer = setInterval (fetchAnimationTimer, 60);
	document.getElementById('temperature').style.opacity = '1.0';
}

function endFetchingAnimation() {
	if (fetchingAnimation.timer !== null) {
		clearInterval (fetchingAnimation.timer);
		fetchingAnimation.timer = null;
	}
	document.getElementById('temperature').style.opacity = '1.0';
}

// blinking the rain probability when it's raining
var rainAlertAnimation = {timer: null, alphaNow:1.0, up:false, duration:1000, start:null};

function rainAlertAnimationTimer() {
	var now = new Date();
	var time = now.getTime();
	var fraction;
	
	if (rainAlertAnimation.up) {
		fraction = (time - rainAlertAnimation.start) / rainAlertAnimation.duration;
	} else {
		fraction = 1.0 - (time - rainAlertAnimation.start) / rainAlertAnimation.duration;
	}
	
	rainAlertAnimation.alphaNow = fraction;
	
	if (rainAlertAnimation.alphaNow > 1.0) {
		rainAlertAnimation.start = time;
		rainAlertAnimation.alphaNow = 1.0;
		rainAlertAnimation.up = false;
	} else if (rainAlertAnimation.alphaNow < 0.0) {
		rainAlertAnimation.start = time;
		rainAlertAnimation.alphaNow = 0.0;
		rainAlertAnimation.up = true;
	}
	
	var sine = Math.sin((Math.PI/2.0) * rainAlertAnimation.alphaNow);
	rainAlertAnimation.alphaNow = sine * sine;
	document.getElementById('rain-probability').style.opacity = rainAlertAnimation.alphaNow;
}

function startRainAlertAnimation() {
	if (rainAlertAnimation.timer !== null) {
		clearInterval (rainAlertAnimation.timer);
		rainAlertAnimation.timer = null;
	}
	var now = new Date();
	rainAlertAnimation.up = false;
	rainAlertAnimation.alphaNow = 1.0;
	rainAlertAnimation.start = now.getTime() - 60;
	rainAlertAnimation.timer = setInterval (rainAlertAnimationTimer, 60);
}

function endRainAlertAnimation() {
	if (rainAlertAnimation.timer !== null) {
		clearInterval (rainAlertAnimation.timer);
		rainAlertAnimation.timer = null;
	}
	document.getElementById('rain-probability').style.opacity = '1.0';
}

// moon phase animation
function showmoonphase() {
	if (!isTiger) {
		document.getElementById('weatherAndMoon').style.webkitTransform = 'rotateY(180deg)';
	} else {
		document.getElementById('weather').style.opacity = 0;
		document.getElementById('moon').style.display = 'block';
	}
	document.getElementById('weatherStationName').style.opacity = 0;
	document.getElementById('sun-moon-icon').src = 'images/moon.png';
}

function hidemoonphase() {
	if (!isTiger) {
		document.getElementById('weatherAndMoon').style.webkitTransform = 'none';
	} else {
		document.getElementById('moon').style.display = 'none';
		document.getElementById('weather').style.opacity = 1.0;
	}
	document.getElementById('weatherStationName').style.opacity = 0.9;
	document.getElementById('sun-moon-icon').src = 'images/sun.png';
}

function nameThatAnimation() {
    document.getElementById('day3').style.webkitAnimationName = 'swallowing';
}

var mdayHidingTimeout, forecastDataDivAdjusting;
function showmore(e) {
	var now = today.getHours();
	var offset = 244 - ((weatherElement.length) * 20 + 144 + 21);
	
	if (isTiger) {
		document.getElementById('forecast-data').style.left = offset + 'px';
	} else {
		document.getElementById('forecast-data').style.WebkitTransform = 'translate(' + offset + 'px,0)';
		if (((0 <= today.getHours()) && (today.getHours() < 6)) || ((17 <= today.getHours()) && (today.getHours() <= 23))) {
      document.getElementById('mday1').addEventListener('webkitTransitionEnd', nameThatAnimation, false);
      mdayHidingTimeout = setTimeout(function(){document.getElementById('mday1').style.width = 0; document.getElementById('mday1').style.opacity = 0;}, 3600);
      forecastDataDivAdjusting = setTimeout(function(){document.getElementById("forecast-data").style.paddingLeft = "24px";}, 3800);
    }
	}
	document.getElementById('legend').style.opacity = 0;
}

function hidemore(e) {
	if (isTiger) {
		document.getElementById('forecast-data').style.left = 0;
	} else {
	  var forecastData = document.getElementById('forecast-data');
		forecastData.style.WebkitTransform = 'translate(0,0)';
		forecastData.style.paddingLeft = 0;
		var mday1 = document.getElementById('mday1');
		mday1.style.width = '24px';
		mday1.style.opacity = 1;
		mday1.removeEventListener('webkitTransitionEnd', nameThatAnimation, false);
		document.getElementById('day3').style.webkitAnimationName = '';
		if (mdayHidingTimeout !== null) {
		  clearTimeout(mdayHidingTimeout);
		  clearTimeout(forecastDataDivAdjusting);
	  }
	}
	document.getElementById('legend').style.opacity = 1;
}